<?php 

/**
 * @file
 * Provides common functionas for cloud_dashboard.module.
 * Works with Cloud Metering Module
 *  
 * Copyright (c) 2011 DOCOMO Innovations, Inc.
 *  
 */



/**
 * Function for configuring Cloud Dashboard settings
 */

function cloud_dashboard_tcpdf_configure() {
  
  $output = drupal_get_form('cloud_dashboard_tcpdf_settings');
  return $output;

}

/**
 * Form for Cloud Dashboard TCPDF Settings using hook_form
 */

function cloud_dashboard_tcpdf_settings() {

  $form['pdf_fieldset'] = array(
    '#type' => 'fieldset',
    '#title' => t('TCPDF Library')
  );
  $form['pdf_fieldset']['cloud_dashboard_pdf_library'] = array(
    '#type' => 'textfield',
    '#title' => t('Path for the TCPDF Library Folder'),
    '#default_value' => variable_get('cloud_dashboard_pdf_library', ''),
    '#description' => t('TCPDF Library is required for generating the PDF Reports from Cloud Dashboard. Please download the TCPDF Library from ' . l("http://www.tcpdf.org", "http://www.tcpdf.org", array("attributes" => array("target" => "_blank"), "html" => TRUE)) . ' and extract it in a folder. Add the folder path where the TCPDF is extracted. e.g. if "sites/all/modules/library/tcpdf" is the folder where TCPDF is extracted, then add the folder path without the trailing slash.') 
  );
  
  $form['pdf_fieldset']['cloud_dashboard_tcpdf_logo'] = array(
    '#type' => 'textfield',
    '#title' => t('Path of the Logo Image'),
    '#default_value' => variable_get('cloud_dashboard_tcpdf_logo', ''),
    '#description' => t('Add the Logo image file path for the adding into the generated Dashboard Report in PDF. e.g. if "sites/all/modules/library/images/pdf_logo.gif" is the path of Logo image, then add the image path without the trailing slash. Image formats JPG and GIF are only allowed.') 
  );
    
  return system_settings_form($form);
}

/**
 * Validating the if the TCPDF folder does exist using hook_validate()
 */

function cloud_dashboard_settings_validate($form, &$form_state) {

  $values = $form_state['values'];
  $folder_path = $values['cloud_dashboard_pdf_library'];
  $logo_path = $values['cloud_dashboard_tcpdf_logo'];
  
  //Check if the folder exist or not
  $get_pdf_library_path = realpath($_SERVER['DOCUMENT_ROOT'] . base_path() . $folder_path);  
  if (!file_exists($get_pdf_library_path)) {
    form_set_error('cloud_dashboard_pdf_library', t('Folder does not exist - %folder_path', array('%folder_path' => $folder_path)));
  }
  
  //Check if the Logo image exist or not
  $get_pdf_logo_path = realpath($_SERVER['DOCUMENT_ROOT'] . base_path() . $logo_path);
  if (!file_exists($get_pdf_logo_path)) {    
    form_set_error('cloud_dashboard_tcpdf_logo', t('Image file does not exist - %logo_path', array('%logo_path' => $logo_path)));
  }
  elseif (!empty($logo_path)) {
    $fileinfo=getimagesize($get_pdf_logo_path);//Getting the image width and height if it is an image file
    $mime_type = $fileinfo['mime'];
    
    if ($mime_type !="image/jpeg" && $mime_type != "image/gif") {
      form_set_error('tcpdf_logo', t('The logo image is not of a valid image format type. Please upload an image of type JPG or GIF.'));
    }
  }
}



/**
 * Function for configuring Cloud Dashboard Date Display Format settings
 */

function cloud_dashboard_date_format_configure() {
  
  $output = drupal_get_form('cloud_dashboard_date_format_settings');
  return $output;

}


/**
 * Form for Cloud Dashboard Date Display Format Settings using hook_form
 */

function cloud_dashboard_date_format_settings() {

  $form['date_fieldset'] = array(
    '#type' => 'fieldset',
    '#title' => t('Date and Time Display Format')
  );
  $form['date_fieldset']['cloud_dashboard_date_format'] = array(
    '#type' => 'textfield',
    '#title' => t('Date Format'),
    '#default_value' => variable_get('cloud_dashboard_date_format', 'Y/m/d'),
    '#description' => t('A user-defined Date format for displaying in Dashboard PopUp. See the ' . l("PHP Manual", "http://php.net/manual/function.date.php", array("attributes" => array("target" => "_blank"), "html" => TRUE)) . ' for more available Date options.'),
    '#size' => 10,
    '#required' => TRUE 
  );
      
  return system_settings_form($form);
}



/**
 * Displaying Metering Dashboard
 */
function cloud_dashboard_display($arg = '') {
  
  //Enabling the javascript for the Charts diagrams   
  drupal_add_js(cloud_get_module_base() . '/js/cloud_dashboard.js');
  
  //Global $user object of Drupal
  global $user;
  
  $display_dashboard = 0;
  
  //Get the Count of the Instances List for Clouds
  $instances_count = cloud_get_all_instances_list_count();
  
  if (!empty($instances_count)) {
    $display_dashboard = 1;
  }
  else {
    $display_dashboard = 0;
  }  
  //Construct the Metering Dashboard page display
  $output  = '<div id="metering_dashboard">';
  
  $output .= '<div class="d_header"><div class="d_header_left">' . t('Metering Dashboard') . '</div><div class="d_header_right">' . t('Welcome: ') . $user->name .'</div></div>';
  $output .= '<div id="container" class="dashboard_container">';
  
  if ($display_dashboard == 1) {
    $output .= _cloud_dashboard_chart_data();
  }
  $output .= '</div>';
  $output .= '<div class="d_tab_container">';
  
  if ($display_dashboard == 1) {
    $output .= drupal_get_form('cloud_dashboard_all_cloud_data');
  }
  elseif ($display_dashboard == 0) {
    $output .= '<h3>' . t('No Cloud Details is Added to display the Dashboard. Please add the Cloud details.') . '</h3>';
  } 
    
  $output .= '</div>';
  
  //Display the output
  return $output;
}


/**
 * Function to display the Cloud Dashboard page
 */

function cloud_dashboard_all_cloud_data() {
  
  global $user;
  //Get the page arguments
  $page_argument = filter_xss(arg(2));
  $page_argument_display = check_plain(filter_xss(arg(3)));
  
  $filters = _cloud_dashboard_get_filters();
  
  if (empty($filters['cloud']) || $filters['cloud'] == 'all') {
   $cloud_list = cloud_get_all_clouds();
  }
  else {
    $cloud_list[$filters['cloud']] = $filters['cloud'];
  }
  
  drupal_add_js(cloud_get_module_base() . 'js/cloud.js', 'module');

  $column      = 'instance_nickname' ;
  
  $form['options'] = array(
    '#type'   => 'fieldset',
    '#prefix' => '<div class="container-inline">',
    '#suffix' => '</div>',
  );
  
  $options = array(
    t('Nickname' ) ,
    t('ID'       ) ,
    //t('Hostname' ) ,
  );
    
  $filter     = $filters['filter_value'] ;
  $filter_col = $filters['filter_column'];  
  
  //Print the PDF Report of the Dashboard Instances listings details  
  if ($filters['op'] == 'Print PDF') {
    _cloud_dashboard_generate_pdf_data($page_argument, $page_argument_display);
  }
  
  if ( $filter_col == 0 )  {
    $column  = 'Nickname'           ;
    $sql_col = 'instance_nickname'  ;
  }
  elseif ($filter_col == 1 ) {
    $column  = 'ID'                 ;
    $sql_col = 'instance_id'        ;
  }
  //elseif ($filter_col == 2 ) {
    //$column  = 'Public-DNS-Name'    ;
    //$sql_col = 'dns_name'           ;
  //}
    
  if ( isset($filter) && !empty($filter) ) {
    
    $query_args[] = $sql_col ;
    $query_args[] = $filter  ;
    
  }
  else {
    
    $query_args[] = '' ;
    $query_args[] = '' ;
    
  }
  
  //Cloud the total instances list    
  $all_instances_count = cloud_get_all_instances_list_count() ;
  $filter_disabled     = '' ;
  if ( $all_instances_count < 2 ) {

    $filter_disabled = TRUE ;
  }
  
  $form['options']['label'    ] = array('#type' => 'item'     , '#title'   => t('Filter'));
  $form['options']['operation'] = array('#type' => 'select'   , '#options' => $options, '#default_value' => $filter_col, '#disabled' => $filter_disabled ) ;
  $form['options']['filter'   ] = array('#type' => 'textfield', '#size'    => 40      , '#default_value' => $filter    , '#disabled' => $filter_disabled ) ;
  $form['options']['submit'   ] = array('#type' => 'submit'   , '#value' => t('Apply')                                 , '#disabled' => $filter_disabled ) ;
       
  //Date filters for Searching between dates of Instances running
  
  if (isset($_REQUEST['date_start_year']) || isset($_REQUEST['date_start'])) {
    if (module_exists('date_popup')) {
      $filter_start_date = isset($_REQUEST['date_start'   ]) ? $_REQUEST['date_start'   ] : '' ;
      $filter_end_date = isset($_REQUEST['date_end'   ]) ? $_REQUEST['date_end'   ] : '' ;
      
      //Convert the Date into the Y-m-d format
      $get_start_date_filter = strtotime($filter_start_date['date']);
      $get_start_date = date('c', $get_start_date_filter);

      $get_end_date_filter = strtotime($filter_end_date['date'] . ' 23:59:59');
      $get_end_date = date('c', $get_end_date_filter);
    }
    else {
      $filter_start_date['year'] = isset($_REQUEST['date_start_year']) ? $_REQUEST['date_start_year'   ] : '' ;
      $filter_start_date['month'] = isset($_REQUEST['date_start_month']) ? $_REQUEST['date_start_month'   ] : '' ;
      $filter_start_date['day'] = isset($_REQUEST['date_start_day']) ? $_REQUEST['date_start_day'   ] : '' ;
  
      $filter_end_date['year'] = isset($_REQUEST['date_end_year']) ? $_REQUEST['date_end_year'   ] : '' ;
      $filter_end_date['month'] = isset($_REQUEST['date_end_month']) ? $_REQUEST['date_end_month'   ] : '' ;
      $filter_end_date['day'] = isset($_REQUEST['date_end_day']) ? $_REQUEST['date_end_day'   ] : '' ;
    
      //Convert the Date into the Y-m-d format    
      $get_start_date_filter = strtotime($filter_start_date['year'] . '-' . $filter_start_date['month'] . '-' . $filter_start_date['day'] );
      $get_start_date = date('c', $get_start_date_filter);

      $get_end_date_filter = strtotime($filter_end_date['year'] . '-' . $filter_end_date['month'] . '-' . $filter_end_date['day'] . ' 23:59:59');
      $get_end_date = date('c', $get_end_date_filter); 
    }
    $start = date('c', strtotime(date('Y', $get_start_date_filter) . '-' . date('m', $get_start_date_filter) . '-' . date('d', $get_start_date_filter) . ' 00:00:00'));
    $end  = date('c', strtotime(date('Y', $get_end_date_filter) . '-' . date('m', $get_end_date_filter) . '-' . date('d', $get_end_date_filter) . ' 23:59:59'));
  }
  else{                             
    $end = date('c');
    $start_period = strtotime($end .' - 29 days');
    $start_month  = strtotime($end .' - 11 months');
    
    if (!$page_argument_display || $page_argument_display == 'daily') {
      $start = date('c', strtotime(date('Y', $start_period) . '-' . date('m', $start_period) . '-' . date('d', $start_period) . ' 00:00:00'));
    }
    elseif ($page_argument_display == 'monthly') {
      //$start = date('c', strtotime(date('Y') . '-01-01 00:00:00'));
      $start = date('c', strtotime(date('Y', $start_month) . '-' . date('m', $start_month) . '-01 00:00:00'));
    }
    elseif ($page_argument_display == 'yearly') {
      $start = _cloud_dashboard_least_runtime_date();
    }
  }
  /*
  else{
    
    if (empty($filters['tab']) || $filters['tab'] == 'daily') {
      $start = date('c', strtotime(date('Y') . '-' . date('m') . '-01 00:00:00'));
    }
    elseif ($filters['tab'] == 'monthly') {
      $start = date('c', strtotime(date('Y') . '-01-01 00:00:00'));
    }
    elseif ($filters['tab'] == 'yearly') {
      $start = _cloud_dashboard_least_runtime_date();
    }
                  
    $end = date('c');
  }
  */
  
  
  //Set the start date as least value of the cloud rumtime value
  $get_least_date = _cloud_dashboard_least_runtime_date();
  $unixtime_value_start = strtotime($get_least_date);  
  
  // ----- Date filter form -----
  // Displaying Date input in form of a Calendar
  if (module_exists('date_popup')) {
    $default_date = date('Y-m-d', strtotime($start));
    $date_format = 'Y-m-d'; 
    $date_type = 'date_popup';
    $default_end = date('Y-m-d');
    $class_date_fieldset = 'container-date_popup';
    
    $form['date_options'] = array(
      '#type'   => 'fieldset',
      '#prefix' => '<div class="' . $class_date_fieldset . '">',
      '#suffix' => '</div>',
    );
    
    //Date filter form  
    $form['date_options']['date_item'] = array(
      '#type' => 'item',
      '#title' => t('Filter Instances by '),
      '#prefix' => '<div class="date_fieldset-title">',
      '#suffix' => '</div>',
    );
    
    $form['date_options']['date_start'] = array('#type' => $date_type, '#title' => t('Start Date'), '#default_value' => $default_date, '#date_format' => $date_format, '#prefix' => '<div class="date_start_input">', '#suffix' => '</div>' );   
    $form['date_options']['date_end'] = array('#type' => $date_type, '#title' => t('End Date'), '#default_value' => $default_end, '#date_format' => $date_format, '#prefix' => '<div class="date_end_input">', '#suffix' => '</div>' );  
  
  }
  else {
    $default_date = array('year' => date('Y', $unixtime_value_start), 'month' => date('n', $unixtime_value_start), 'day' => date('j', $unixtime_value_start) ); 
    $date_type = 'date';
    $date_format = '';
    $default_end = array('year' => date('Y'), 'month' => date('n'), 'day' => date('j') ); 
    $class_date_fieldset = 'container-inline';
    
    $form['date_options'] = array(
      '#type'   => 'fieldset',
      '#prefix' => '<div class="' . $class_date_fieldset . '">',
      '#suffix' => '</div>',
    );
    
    //Date filter form  
    $form['date_options']['date_item'] = array(
      '#type' => 'item',
      '#title' => t('Filter Instances by '),
      '#prefix' => '<div class="date_fieldset-title">',
      '#suffix' => '</div>',
    );
  
    //Year options
    for ($i = date('Y', $unixtime_value_start); $i <= date('Y'); $i++) {
      $options_year[$i] = $i;
    }
  
    //Month options
    for ($i = 1; $i <= 12; $i++) {
      $options_months[$i] = $i;
    }
  
    //Month options
    for ($i = 1; $i <= 31; $i++) {
      $options_days[$i] = $i;
    }
  
    //Creating the Date filter fields
    $form['date_options']['date_start_year'] = array('#type' => 'select', '#title' => t('Start Date'), '#options' => $options_year, '#default_value'  => date('Y', strtotime($start)) );
    $form['date_options']['date_start_month'] = array('#type' => 'select', '#options' => $options_months, '#default_value'  => date('n', strtotime($start)) );
    $form['date_options']['date_start_day'] = array('#type' => 'select', '#options' => $options_days, '#default_value'  => date('j', strtotime($start)) );
  
  
    //Creating the Date filter fields
    $form['date_options']['date_end_year'] = array('#type' => 'select', '#title' => t('End Date'), '#options' => $options_year, '#default_value'  => date('Y') );
    $form['date_options']['date_end_month'] = array('#type' => 'select', '#options' => $options_months, '#default_value'  => date('n') );
    $form['date_options']['date_end_day'] = array('#type' => 'select', '#options' => $options_days, '#default_value'  => date('j') );
  
    //End of creating the Date filter fields
  }
  
  $form['date_options']['submit'] = array(
    '#type' => 'submit',
    '#value' => t('Filter'),
    '#prefix' => '<div class="date_fieldset-button">',
    '#suffix' => '</div>',
  );
  //End of Date filters
  // --- Date filter form ends here ---
  
  
  //Button to generate PDF Report
  if (user_access('Print Cloud Dashboard PDF Report')) {
    $form['generate_pdf'] = array('#type' => 'submit', '#value' => t('Print PDF'));
  }
  
  
  $form['header'] = array(
  
    '#type' => 'value',  
    '#value' => array(
      array('data' => t('Nickname'), 'field' => 'instance_nickname'
                                   , 'class' => 'nickname-column'  ), 
      array('data' => t('ID'      ), 'field' => 'instance_id'      ), 
      array('data' => t('Hostname'), 'field' => 'dns_name'         ), 
      array('data' => t('Cost'   ),  'field' => '', 'sort' => 'asc'),             
      array('data' => t('Action'  ), 'class' => 'action-column'    )
    )
    
  );
  
  $default_sorting = " ORDER BY instance_state_name ASC";
      
  $all_instances_list = array() ;
  $filter_params = array( 'column' => $sql_col, 'filter_value' => $filter ) ;
    
  $sorting_request = isset($_REQUEST['order']) ? filter_xss($_REQUEST['order']) : '' ;
  $sorting_order   = isset($_REQUEST['sort']) ? filter_xss($_REQUEST['sort']) : 'ASC' ;
    
  $header_sorting = $form['header']['#value'];
  
  foreach ($header_sorting as $value) {
    if ($value['data'] == $sorting_request) {
      $table_sort = " ORDER BY instance_state_name ASC, " . $value['field'] . " " . $sorting_order;
    }  
  }
  
  //The main query for fetching the instances  
  $query = _cloud_dashboard_generate_instance_data_query($cloud_list) ;
  
  if (!empty($sorting_request) && $sorting_request != 'Cost') {
    $query .= $table_sort;
  }
  elseif (empty($sorting_request) || $sorting_request == 'Cost') {
    $query .= $default_sorting . ', instance_nickname ' . $sorting_order;
  }
  
  $result = pager_query( $query, CLOUD_PAGER_LIMIT, 0, NULL, $query_args );
  
  $instances_list_data = array();
  
  while ( $instance = db_fetch_array($result) ) {
  
    $cloud_context       = $instance['cloud_type'] ;
    $key                 = $instance['instance_id'  ] ;      
    $runtime             = $instance['launch_time'  ] ;  
    $unit_linux          = $instance['linux_or_unix_usage'];
    $unit_windows        = $instance['windows_usage'];    
    $terminated_date     = $instance['terminated_date'];
  
    $total_cost_instance = _cloud_dashboard_instance_total_cost($start, $end, $runtime, $terminated_date, $unit_linux);
         
    //Create the Instances Array
    $instances_list_data[$key] = $instance;
    $instances_list_data[$key]['total_cost'] = $total_cost_instance['total_cost'];
    
    $all_instances_list = array_merge( $all_instances_list , $instances_list_data ) ;
    
  }
    
  //Separate all running and Terminated Instances
  $all_running_instances = array();
  $all_terminated_instances = array();
  
  foreach ($all_instances_list as $key => $value) {     
     if (empty($value['terminated_date'])) {
       $all_running_instances[$key] = $value;
     }
     elseif (!empty($value['terminated_date'])) {
       $all_terminated_instances[$key] = $value;
     } 
  }  
   
  
  $ts = tablesort_init( $form['header']['#value'] );
    
  if ($ts['name']) {
    
    if ($ts['sql'] === 'instance_id' ) {

      $field = 'instance_id' ;
    }
    elseif ($ts['sql'] === 'instance_nickname' ) {

      $field = 'name' ;
    }
    elseif ($ts['sql'] === 'dns_name' ) {

      $field = 'hostname' ;
    }
    elseif ($ts['sql'] === 'costing' || $ts['sql'] === '' ) {

      $field = 'costing' ;
    }
    else {
    
      $field = 'id' ;
    }
    
    //Sort and search all Running Instances    
    $all_running_instances_data = _cloud_dashboard_create_listing($all_running_instances, $field, $ts['sort'], $filter, $sql_col);
    
    //Sort and search all Terminated Instances
    $all_terminated_instances_data = _cloud_dashboard_create_listing($all_terminated_instances, $field, $ts['sort'], $filter, $sql_col);
    
  }
  //All Running Instances  
  foreach ($all_running_instances_data as $instance) {
      
    //Get the Instance Object details from Cloud API
    //$instance_value = cloud_get_instance($instance['cloud_type'], $instance['instance_id']) ;
        
    // Some modules like XCP does not implement hook_cloud_get_instance()
    $instance_value = cloud_metering_get_instance($instance['cloud_type'], $instance['instance_id']);
    
    $cloud_context       = $instance['cloud_type'] ;
    $key                 = $instance['instance_id'  ] ;
    $name                = trim($instance_value['name'    ]);
    $state               = $instance_value['state'        ] ;   
    $user_key            = $instance_value['user'         ] ; 
    $hostname            = $instance_value['hostname'     ] ; 
    $zone                = $instance_value['zone'         ] ;  
    $runtime             = $instance['launch_time'        ] ;  
    $lock_status         = isset( $instance_value['lock_status'  ] ) ? $instance_value['lock_status'  ] : FALSE ;
    $unit_linux          = $instance['linux_or_unix_usage'];
    $unit_windows        = $instance['windows_usage'];    
    $terminated_date     = $instance['terminated_date'];
    $total_cost_instance = $instance['total_cost'];
    
    $total_cost = t( '$' . number_format($total_cost_instance, 2));
    
    
      $cloud_menu_path = cloud_get_menu_path($cloud_context ) ;
            
      //Get the Cloud instance Popup details
      $instance_popup_data = _cloud_dashboard_instance_popup($instance);
      
      $nick_name = substr( $name, 0, strpos($name,  '.') ) ;
      if ( strlen($nick_name) > 3 ) { // truncate only in case the name is long  
        $nick_name = substr($name, 0, strpos($name,  '.')) . '...' ;
      }
      else {
        $nick_name = $name ;
      }
      
      if (!empty($name)) {
        $form['Nickname'][$key] =  array(
          array(
            '#value' => l( $nick_name, $cloud_menu_path . '/instances/describe/' . $key . '/info', array('attributes' => array('target' => '_blank'), 'query' => 'instance_id=' . urlencode($key))) . ( ($lock_status == 'Lock') ? '<a class=lockIcon title=Lock>&nbsp;&nbsp;&nbsp;&nbsp;</a>' : '' ) . '<div style="height: 20px;" class="' . $key . '"></div>')  // end of array
        ); // end of array
      }
      else {
        if ($instance['instance_nickname']) {
          $nickname = $instance['instance_nickname'];
        }
        else {
          $nickname = $key;
        }
        $form['Nickname'][$key] = array(array('#value' => '<div style="height: 20px;" class="' . $key . '">' . $nickname . '</div>' )) ;
      }
      
      $instance_idFormatted = (strlen($key) > 10)
                          ? substr($key, 0, 8) . '...'
                          : $key ;
      $form['ID'        ][$key] = array( array('#value' => '<div class="' . $key . '">' . $instance_idFormatted . '</div>' ) );
      
      $public_dns = substr( $hostname, 0, strpos($hostname,  '.') ) ;
      if ( strlen($public_dns) > 3 ) { // truncate only in case the name is long
        $public_dns = substr($hostname, 0, strpos($hostname,  '.')) . '...' ;
      }
      else {
        $public_dns = $hostname ;
      }
     
      if (!empty($hostname)) { 
        $form['Public_DNS'][$key] = array( array('#value' =>  '<div class="' . $key . '">' . l($public_dns, 'http://' . $hostname, array('attributes' => array('target' => '_blank'), 'query' => ''))  . '</div>' ) );
      }
      else {
        $form['Public_DNS'][$key] = array( array('#value' =>  '<div class="' . $key . '">Instance ' . ucwords($instance['instance_state_name']) . '</div>' ) );
      }
      
      $form['instance_cost'][$key] = array( array('#value' => '<div class="' . $key . '">' . $total_cost . '</div>' ) );
      
      $form['cloud_item'][$key] = array(
            '#prefix' => '<div class="cloud_item_container" id="' . $key . '" style="display: none">',
            '#type' => 'item',
            '#value' => $instance_popup_data,
            '#suffix' => '</div>',
      );
  }
  
  
  //All Terminated Instances  
  foreach ($all_terminated_instances_data as $instance) {
      
    //Get the Instance Object details from Cloud API
    //$instance_value = cloud_get_instance($instance['cloud_type'], $instance['instance_id']) ;
    
    // Some modules like XCP does not implement hook_cloud_get_instance()
    $instance_value = cloud_metering_get_instance($instance['cloud_type'], $instance['instance_id']);
    
    $cloud_context       = $instance['cloud_type'] ;
    $key                 = $instance['instance_id'  ] ;
    $name                = trim($instance_value['name'    ]);
    $state               = $instance_value['state'        ] ;   
    $user_key            = $instance_value['user'         ] ; 
    $hostname            = $instance_value['hostname'     ] ; 
    $zone                = $instance_value['zone'         ] ;  
    $runtime             = $instance['launch_time'      ] ;  
    $lock_status         = isset( $instance_value['lock_status'  ] ) ? $instance_value['lock_status'  ] : FALSE ;
    $unit_linux          = $instance['linux_or_unix_usage'];
    $unit_windows        = $instance['windows_usage'];    
    $terminated_date     = $instance['terminated_date'];
    $total_cost_instance = $instance['total_cost'];
    
    $total_cost = t( '$' . number_format($total_cost_instance, 2));
    
    
      $cloud_menu_path = cloud_get_menu_path($cloud_context ) ;
            
      //Get the Cloud instance Popup details
      $instance_popup_data = _cloud_dashboard_instance_popup($instance);
      
      $nick_name = substr( $instance['instance_nickname'], 0, strpos($instance['instance_nickname'],  '.') ) ;
      if ( strlen($nick_name) > 3 ) { // truncate only in case the name is long  
        $nick_name = substr($instance['instance_nickname'], 0, strpos($instance['instance_nickname'],  '.')) . '...' ;
      }
      else {
        $nick_name = $instance['instance_nickname'] ;
      }
      
      if ($instance['instance_nickname']) {
        $nickname = $nick_name;
      }
      else {
        $nickname = $key;
      }
      $form['Nickname'][$key] = array(array('#value' => '<div style="height: 20px;" class="' . $key . '">' . $nickname . '</div>' )) ;
      
      $instance_idFormatted = (strlen($key) > 10)
                          ? substr($key, 0, 8) . '...'
                          : $key ;
      
      $form['ID'        ][$key] = array( array('#value' => '<div class="' . $key . '">' . $instance_idFormatted . '</div>' ) );
      $form['Public_DNS'][$key] = array( array('#value' =>  '<div class="' . $key . '">Instance Terminated</div>' ) );
      $form['instance_cost'][$key] = array( array('#value' => '<div class="' . $key . '">' . $total_cost . '</div>' ) );
      
      $form['cloud_item'][$key] = array(
            '#prefix' => '<div class="cloud_item_container" id="' . $key . '" style="display: none">',
            '#type' => 'item',
            '#value' => $instance_popup_data,
            '#suffix' => '</div>',
      );
  }  
  
  $form['pager'    ] = array('#value' => theme('pager', NULL, CLOUD_PAGER_LIMIT, 0));
  $form['#redirect'] = FALSE;
  
  return $form ; 

}


/**
 * Implementing hook_form_validate()
 */
function cloud_dashboard_all_cloud_data_validate($form_id, $form_values) {
  $values = $form_values['values'];
  $filter = $values['filter'];
    
  //Get the page arguments
  $page_argument = filter_xss(check_plain(arg(3)));
  
  if (empty($page_argument)) {
    $page_argument = 'daily';
  }
  
  if (module_exists('date_popup')) {
    $filter_start_date = $values['date_start'];
    $filter_end_date = $values['date_end'];
      
    $get_start_date_filter = strtotime($filter_start_date);
    $get_start_date = date('c', $get_start_date_filter);
  
    $get_end_date_filter = strtotime($filter_end_date);
    $get_end_date = date('c', $get_end_date_filter);
  }
  else {
    $filter_start_date['year'] = $values['date_start_year'];
    $filter_start_date['month'] = $values['date_start_month'];
    $filter_start_date['day'] = $values['date_start_day'];
      
    $filter_end_date['year'] = $values['date_end_year'];
    $filter_end_date['month'] = $values['date_end_month'];
    $filter_end_date['day'] = $values['date_end_day'];
    
    $get_start_date_filter = strtotime($filter_start_date['year'] . '-' . $filter_start_date['month'] . '-' . $filter_start_date['day'] );
    $get_start_date = date('c', $get_start_date_filter);
  
    $get_end_date_filter = strtotime($filter_end_date['year'] . '-' . $filter_end_date['month'] . '-' . $filter_end_date['day']);
    $get_end_date = date('c', $get_end_date_filter);
            
    //Check the Start Date filter agianst valid Date 
    if (!checkdate($filter_start_date['month'], $filter_start_date['day'], $filter_start_date['year'])) {
      form_set_error('error', t('Invalid Start Date. Please select a valid Start Date.'));
      drupal_goto('reports');
    }
    //Check the End Date filter agianst valid Date  
    if (!checkdate($filter_end_date['month'], $filter_end_date['day'], $filter_end_date['year'])) {
      form_set_error('error', t('Invalid End Date. Please select a valid End Date.'));
      drupal_goto('reports');
    }    
  }
    
  $operation = $_REQUEST['op'];  
  
  if ($operation == 'Apply') {
    if (empty($filter)) {
      form_set_error('filter', t('Please enter values in the Search Textfield.'));
      drupal_goto($_REQUEST['q']);
    }
  }
  if ($operation == 'Filter') {
        
    //Check the if Start Date is greater than End Date 
    if ($get_start_date_filter > $get_end_date_filter) {
      form_set_error('error', t('Start Date cannot be greater than End Date.'));
      drupal_goto('reports');
    }
    
    //$day_diff = ($get_end_date_filter - $get_start_date_filter) / (60 * 60 * 24);

    $date_diff = _cloud_dashboard_date_diff($get_end_date, $get_start_date);
    $month_diff = $date_diff['months_total'];
    $day_diff   = $date_diff['days_total'];
       
    if ($page_argument == 'monthly' && $month_diff > 11) {
      form_set_error('error', t('Please select the Start Date and End Date within 12 Months.'));
      drupal_goto($_REQUEST['q']);
    }
  
    if ($page_argument == 'daily' && $day_diff > 30) {
      form_set_error('error', t('Please select the Start Date and End Date within 30 Days.'));
      drupal_goto($_REQUEST['q']);
    }
    /*
    elseif ($page_argument == 'daily' && date('m', $get_start_date_filter) != date('m', $get_end_date_filter)) {
      form_set_error('error', t('Please select the Start Date and End Date within the same Month of the Year in Daily View.'));
      drupal_goto($_REQUEST['q']);
    }
    */
  }
}

/**
 * Implementing hook_form_submit()
 */

function cloud_dashboard_all_cloud_data_submit($form_id, $form_values) {
  $values = $form_values['values'];
}


/**
 * Implementing hook_theme() for the form
 */
 
function cloud_dashboard_theme() {
  return array(
    'cloud_dashboard_all_cloud_data' => array('arguments' => array('form' => NULL)),
  );
}

function theme_cloud_dashboard_all_cloud_data($form) {
  
  $output = '';
  
  global $user;
  
  //Page arguments
  $page_argument = filter_xss(arg(2));
  $display_argument = filter_xss(arg(3));
    
  $display_tab = !empty($display_argument) ? $display_argument : '';    
    
  //Get the list of all Cloud instances
  $cloud_list = cloud_get_all_clouds();
  
  $active_class = '';
  if (!$page_argument || $page_argument == 'all') {
   $cloud_list = cloud_get_all_clouds();
   $active_class = 'active';
  }
  else {
    $cloud_list[$page_argument] = $page_argument;    
  }
  
  //Get the names of the Cloud instances
  $cloud_name_list = cloud_get_all_clouds_display_name();
  
  array_multisort($cloud_name_list, SORT_ASC, $cloud_list);
  
  $list_data = '<div id="tabs-wrapper">';
  $list_data .= t('<ul class="tabs primary" style="float: none">');
  $list_data .= t('<li style="display: inline" class = "' . $active_class . '">' . l(t('All'), 'reports/cloud_dashboard/all/' . $display_tab , array('attributes' => array('class' => $active_class ))) . '</li>');
  foreach ($cloud_list as $cloud_context) {
    if ($cloud_context == $page_argument) {
      $active_cloud = 'active';
    }
    else {
      $active_cloud = '';
    }
    
    $list_data .= t('<li style="display: inline" class ="' . $active_cloud . '">' . l($cloud_name_list[$cloud_context], 'reports/cloud_dashboard/' . $cloud_context . '/' . $display_tab, array('attributes' => array('class' => $active_cloud ))) . '</li>');
    $get_cloud_name[] = $cloud_context;
  }
  $list_data .= t('</ul></div>');
  
  $form['list_item'] = array(
    '#type' => 'item',
    '#value' => $list_data
  );
   
    
  //Add the Cloud Tabs
  $output .= drupal_render($form['list_item']);
    
    if ( isset($form['options']['filter']['#default_value']) ) {
  
        $filter_col = $form['options']['operation']['#default_value'] ;
        if ( $filter_col == 0 )  {
            $sql_col = 'instance_nickname' ;
        }
        elseif ($filter_col == 1 ) {
            $sql_col = 'instance_id' ;
        }
        elseif ($filter_col == 2 ) {
            $sql_col = 'dns_name' ;
        }
  
        $filter = $form['options']['filter']['#default_value'] ;
        $filter_params = array( 'column' => $sql_col, 'filter_value' => $filter ) ;
    }
    else {
  
        $filter_params = array() ;
    }
  
  $all_instances_list = cloud_get_all_instances_list($filter_params) ;
    
  $form['Nickname'] = !empty($form['Nickname']) ? $form['Nickname'] : array();
  $form['pager']['#value'] = !empty($form['pager']['#value']) ? '<div class="clear-block">' . $form['pager']['#value'] . '</div>' : '';
  $rows = array();
  
  foreach (element_children($form['Nickname']) as $key) {
  
    $cloud_name = $form['Nickname'][$key][0]['#value'];
    
    $row = array();
    $lock_image = '';
      
    $row[] = array('data' => drupal_render($form['Nickname'][$key]) . $lock_image, 'class' => 'nickname-column');
    $row[] = drupal_render($form['ID'        ][$key]) ;
    $row[] = drupal_render($form['Public_DNS'][$key]) ;
    $row[] = drupal_render($form['instance_cost'][$key]) ;    
    
    if (!empty($all_instances_list[$key])) {
      $action_data = $all_instances_list[$key]['action_data'] ;
    }
    else {
      $action_data = '';
    }
    
    $row[] =  array('data' => $action_data, 'class' => 'action-column' );
    $rows[] = $row;    

  }
  
  $table_attr = array() ;
  $table_attr['id'] = 'all_instances_list_table' ;
  $table_attr['autoupdate_url'] = url( 'reports/callback_get_all_instances_list' ) ;

  $output .= theme('table', $form['header']['#value'], $rows, $table_attr );

   
  if ($form['pager']['#value']) {
    $output .= drupal_render($form['pager']);
  }
  //Add the Date filters
  $output .= drupal_render($form['date_options']);
  $output .= drupal_render($form['options']);
  $output .= drupal_render($form['generate_pdf']);
  $output .= drupal_render($form);
 
  return $output; 
}


/**
 * Function for getting the earliest launch_date among all or specific cloud's instances
 */
 
function _cloud_dashboard_least_runtime_date( $cloud_context = NULL ) {  
  
  $cloud_list = empty($cloud_context) ? cloud_get_all_clouds() : array( $cloud_context => $cloud_context );
  
  $filter_params = array();
  $all_instances_list = array();
  
  foreach ($cloud_list as $cloud_context) {    
    $instances_list     = cloud_get_instances( $cloud_context, $filter_params );
    $all_instances_list = array_merge( $all_instances_list , $instances_list );
  }
  
  $instances_having_runtime = array();
  
  foreach ($all_instances_list as $key => $row) {
    if (!empty($row['runtime'])) {
      $runtime[$key] = $row['runtime'] ;
      $instances_having_runtime[$key] = $all_instances_list[$key];
    }
  }
  
  array_multisort($runtime, SORT_ASC , $instances_having_runtime);
  
  $runtime_data = array();
  
  foreach ($instances_having_runtime as $key => $value) {
    $runtime_data[] = date('c', strtotime($value['runtime'])) ;
  }
  
  return $runtime_data[0];
}

/**
 *  Get the TCPDF configurations
 *  @return {mixed} If TCPDF has been configured, returns the configurations
 *                  Otherwise returns FALSE
 */
function _cloud_dashboard_tcpdf_config() {
  
  $path_cofig  = variable_get('cloud_dashboard_pdf_library', '');
  $logo_config = variable_get('cloud_dashboard_tcpdf_logo',  '');
  
  $real_path   = realpath($_SERVER['DOCUMENT_ROOT'] . base_path() . $path_cofig);
    
  if (!empty($path_cofig) && file_exists($real_path)) {
    
    return array(
      'tcpdf_path'    => $real_path,
      'pdf_logo_path' => $logo_config,
    );
    
  }
  
  return FALSE;
  
}

/**
 * Function get the Total runtime hours difference between two different dates
 * @param $launch_time = Start Date
 * @param $curr_time = End Date, which can be also empty
 */
function _cloud_dashboard_get_runtime($launch_time, $curr_time = '') {

  if (empty($launch_time)) {
    return 0 ;
  }
  
  $launch_tmp_time = date_parse($launch_time) ;
  $gm_launch_time  = gmmktime( $launch_tmp_time['hour'] , $launch_tmp_time['minute'] , $launch_tmp_time['second'] , $launch_tmp_time['month'], $launch_tmp_time['day'], $launch_tmp_time['year'] ) ;
  
  if (!empty($curr_time)) {
    $curr_tmp_time = date_parse($curr_time) ;
    $gm_current_time  = gmmktime( $curr_tmp_time['hour'] , $curr_tmp_time['minute'] , $curr_tmp_time['second'] , $curr_tmp_time['month'], $curr_tmp_time['day'], $curr_tmp_time['year'] ) ;
  }
  else {
    $gm_current_time = gmmktime() ;
  }
  
  $diff =  $gm_current_time - $gm_launch_time ;

  $days = $diff / (86400) ;
  $hr   = $diff /  (3600) ;
  $min  = $diff /    (60) ;
  $sec  = $diff ;

  $hours = 0;
  if ( round( $days) > 0 ) {
    $hours = $days * 24;
  }
  elseif ( round( $hr )  > 0 ) {
    $hours = $hr;
  }
  elseif ( round( $min )  > 0 ) {
    $hours = $min / 60;
  }
  elseif ( round( $sec ) > 0 ) {
    $hours = $sec / 3600;
  }
  return round($hours) ;
}

/**
 * 
 * Converting the Instance date into UTC/GTM time
 * @param $date
 */

function _cloud_dashboard_instance_convert_date($date_input) {
  
  $date_tmp = strtotime($date_input);
            
  //Convert the date into UTC/GMT time
  $get_date = strtotime(date('c', gmdate($date_tmp)));
  $converted_date = date('c', $get_date);
  
  return $converted_date;
}


/**
 * 
 * Function to Generate the Cloud Instances details with Termianted Date
 * Calculate the Total runtime cost of the Instances along with Terminated Instances
 * @param $instances_list = Instance details array.
 */

function _cloud_dashboard_get_instances_details($instances_list) {
  //Set the time to UTC/GMT
  //date_default_timezone_set('UTC');

  //Get the page argument for Tab display
  $page_argument_display = check_plain(filter_xss(arg(3)));
  
  $instances_list_data = array();

  //Validate the instances Terminated Date if it exist when Cloud API is not returning the Instance details
  foreach ($instances_list as $key => $value) {
        
    if (!empty($value['runtime'])) {
        
      //Get the Instance Object details from Cloud API
      $row = cloud_get_instance($value['cloud_context'], $key);
      
      if (isset($_REQUEST['date_start_year']) || isset($_REQUEST['date_start'])) {
        if (module_exists('date_popup')) {
          $filter_start_date = isset($_REQUEST['date_start'   ]) ? $_REQUEST['date_start'   ] : '' ;
          $filter_end_date = isset($_REQUEST['date_end'   ]) ? $_REQUEST['date_end'   ] : '' ;
          
          //Convert the Date into the Y-m-d format
          $get_start_date_filter = strtotime($filter_start_date['date']);
          $get_start_date = date('c', $get_start_date_filter);
  
          $get_end_date_filter = strtotime($filter_end_date['date'] . ' 23:59:59');
          $get_end_date = date('c', $get_end_date_filter);
        }
        else {
          $filter_start_date['year'] = isset($_REQUEST['date_start_year']) ? $_REQUEST['date_start_year'   ] : '' ;
          $filter_start_date['month'] = isset($_REQUEST['date_start_month']) ? $_REQUEST['date_start_month'   ] : '' ;
          $filter_start_date['day'] = isset($_REQUEST['date_start_day']) ? $_REQUEST['date_start_day'   ] : '' ;
      
          $filter_end_date['year'] = isset($_REQUEST['date_end_year']) ? $_REQUEST['date_end_year'   ] : '' ;
          $filter_end_date['month'] = isset($_REQUEST['date_end_month']) ? $_REQUEST['date_end_month'   ] : '' ;
          $filter_end_date['day'] = isset($_REQUEST['date_end_day']) ? $_REQUEST['date_end_day'   ] : '' ;
        
          //Convert the Date into the Y-m-d format    
          $get_start_date_filter = strtotime($filter_start_date['year'] . '-' . $filter_start_date['month'] . '-' . $filter_start_date['day'] );
          $get_start_date = date('c', $get_start_date_filter);
  
          $get_end_date_filter = strtotime($filter_end_date['year'] . '-' . $filter_end_date['month'] . '-' . $filter_end_date['day'] . ' 23:59:59');
          $get_end_date = date('c', $get_end_date_filter); 
        }
        $start = date('c', strtotime(date('Y', $get_start_date_filter) . '-' . date('m', $get_start_date_filter) . '-' . date('d', $get_start_date_filter) . ' 00:00:00'));
        $end  = date('c', strtotime(date('Y', $get_end_date_filter) . '-' . date('m', $get_end_date_filter) . '-' . date('d', $get_end_date_filter) . ' 23:59:59'));
      }
      else{                             
        $end = date('c');
        $start_period = strtotime($end .' - 29 days');
        $start_month  = strtotime($end .' - 11 months');
         
        if (!$page_argument_display || $page_argument_display == 'daily') {
          $start = date('c', strtotime(date('Y') . '-' . date('m', $start_period) . '-' . date('d', $start_period) . ' 00:00:00'));
        }
        elseif ($page_argument_display == 'monthly') {
          //$start = date('c', strtotime(date('Y') . '-01-01 00:00:00'));
          $start = date('c', strtotime(date('Y', $start_month) . '-' . date('m', $start_month) . '-01 00:00:00'));
        }
        elseif ($page_argument_display == 'yearly') {
          $start = $value['runtime'];
        }
      }
      if (time() < strtotime($end)) {
        $end = date('c');
      }
      
      //Get the Instance Total Cost and runtime hours
      $get_costing_hours = _cloud_dashboard_instance_total_cost($start, $end, $value['runtime'], $value['terminated_date'], $value['linux_or_unix_usage']);      
      
      //ADd the total hours into the Array
      $value['total_hours'] = $get_costing_hours['total_hours'];
      
      //Add the total cost into the Array
      $value['total_cost'] = $get_costing_hours['total_cost'];
      //drupal_set_message($value['instance_id'] . '--' . $start . '--' . $end . '--' . $value['total_cost']);
      $ssh_key = $row['key_name'];
      $hostname = $row['hostname'];
      
      $value['key_name'] = $ssh_key;
      $value['hostname'] = $hostname;
      //print $value['instance_id'].' '. $get_costing_hours['total_hours'] . ' ' . $get_costing_hours['total_cost'] .'<br/>';
      $instances_list_data[] = $value;          
    }
  }
  
  return $instances_list_data;
}

/**
 * 
 * Function to get the total hours running of each Instance
 * Get the total cost of each Instance running
 * @param $start = Start Date
 * @param $end = End Date
 * @param $runtime = Instance Launcing date
 * @param $terminated = Instance Terminated Date
 * @param $unit_price = Intance Unit Price
 */


function _cloud_dashboard_instance_total_cost($start, $end, $runtime, $terminated, $unit_price) {
  $return_data = array();
  
  $start_date = gmdate('c', strtotime($start));
  $end_date   = gmdate('c', strtotime($end));
      
  $instance_launch_date     = gmdate('c', strtotime($runtime));
  
  if (!empty($terminated)) {
    $instance_terminated_date = gmdate('c', strtotime($terminated));
  }

    
  if (strtotime($start_date) < strtotime($instance_launch_date)) {
    $start_date = $instance_launch_date;
  }

  if (strtotime($end_date) > strtotime(gmdate('c'))) {
    $end_date = gmdate('c');
  }
    
  if (!empty($terminated) && strtotime($end_date) >= strtotime($instance_terminated_date)) {
    $end_date = $instance_terminated_date;
  }
    
  $total_hours = _cloud_dashboard_get_runtime($start_date, $end_date);
       
  $total_instance_type_charge = ($total_hours > 0 ) ? ($total_hours * $unit_price) : 0;
  $total_cost = $total_instance_type_charge;
        
  $return_data['total_cost']  = $total_cost;
  $return_data['total_hours'] = $total_hours;
  
  return $return_data;
}


/**
 * 
 * Function that Sorts and Search all Instances
 * @param Array $all_instances
 * @param String $field
 * @param Array $ts
 * @param String $filter
 * @param String $sql_col
 */

function _cloud_dashboard_create_listing($all_instances, $field, $ts, $filter, $sql_col) {
  
  //Sorting of Running Instances
  $sort_by_key = array() ;
  $zone        = array();
  $nickname    = array();
  $costing     = array();
      
  foreach ($all_instances as $key => $instance_data) {
    
    //Get the Instance Object details from Cloud API
    $row = cloud_metering_get_instance($instance_data['cloud_type'], $key);
    
    if (!empty($row) && $field != 'costing') {
      $sort_by_key[$key]  = drupal_strtolower($row[$field]); 
    }
    else {
      if (array_key_exists($field, $instance_data)) {
        $sort_by_key[$key]  = drupal_strtolower($instance_data[$field]);
      }
      else {
        $sort_by_key[$key]  = drupal_strtolower($instance_data['instance_id']);
      } 
    }
    
    $zone[$key]         = drupal_strtolower($row['zone']);    
    $nickname[$key]     = drupal_strtolower($instance_data['instance_nickname']); 
    $costing[$key]      = $instance_data['total_cost'];
        
    $all_instances[$key]['total_cost'] = $instance_data['total_cost'];
            
    $launch_tmp_time = date_parse($row['runtime']) ;

    $instance_nickname   = !empty($instance_data['instance_nickname']) ? $instance_data['instance_nickname'] : $key;
    $nick_name_check     = !empty($row['name']) ? $row['name'] : $instance_nickname;
      
    if (!empty($filter)) {
      if ($sql_col == 'instance_nickname') {
        $column_check = $nick_name_check;
      }
      elseif ($sql_col == 'instance_id') {
        $column_check = $key;
      }
        
      if ($filter !== $column_check) {
        unset($all_instances[$key]);
      }
    }
  }
    
  if (empty($zone) === FALSE )  {
    // This is the deafult sort
    if ($field === 'zone' && $ts === 'asc') {
      array_multisort($zone, SORT_ASC, $nickname, SORT_ASC, $runtime, SORT_DESC, $all_instances);
    }
    else {
      if ($field === 'costing') {
        $sort_by_key = $costing ;
      }
      
      $sort_order = SORT_ASC ;

      if ($ts === 'desc') {
        $sort_order = SORT_DESC ;
      }
        
      //Sort the array as per user selection from the Table header
      if (empty($filter)) {
        array_multisort($sort_by_key, $sort_order, $nickname, SORT_ASC, $all_instances, SORT_ASC);
      }
    }
  }
    
  return $all_instances;
}

/**
 *  Get the filters set for Dashboard
 *  Abhijit - 8-23-2011
 */
function _cloud_dashboard_get_filters() {
  
  $filters = array();
  
  // The tab selected
  $tab = filter_xss(arg(3));
  if (!empty($tab)) {
    $tab = _cloud_metering_filter_input($tab);
  }
  else {
    $tab = 'daily';
  }
  $filters['tab'] = $tab;
  
  // The cloud selected
  $cloud = filter_xss(arg(2));
  if (!empty($cloud)) {
    $cloud = _cloud_metering_filter_input($cloud);
  }
  else {
    $cloud = 'all';
  }
  $filters['cloud'] = $cloud;
  
  $filter_op  = isset($_REQUEST['op']) ? _cloud_metering_filter_input($_REQUEST['op']) : '';
  
  $start_date    = '';
  $end_date      = '';
  $filter_column = 0;
  $filter_value  = '';
  
  
  if (!empty($filter_op)) {
    
    $filters['op'] = $filter_op;
    
    switch ($filter_op) {
    
      case 'Filter':
        
        if ( module_exists('date_popup') ) {
          
          $start_date = isset($_REQUEST['date_start'   ]) ? _cloud_metering_filter_input($_REQUEST['date_start'   ]['date']) : '' ;
          $end_date   = isset($_REQUEST['date_end'     ]) ? _cloud_metering_filter_input($_REQUEST['date_end'     ]['date']) : '' ;
          
        }
        else {
          
          $start_year  = isset($_REQUEST['date_start_year']) ? _cloud_metering_filter_input($_REQUEST['date_start_year'   ] )  : '' ;
          $start_month = isset($_REQUEST['date_start_month']) ? _cloud_metering_filter_input($_REQUEST['date_start_month'   ]) : '' ;
          $start_day   = isset($_REQUEST['date_start_day']) ? _cloud_metering_filter_input($_REQUEST['date_start_day'   ]    ) : '' ;
      
          $end_year    = isset($_REQUEST['date_end_year']) ? _cloud_metering_filter_input($_REQUEST['date_end_year'   ])   : '' ;
          $end_month   = isset($_REQUEST['date_end_month']) ? _cloud_metering_filter_input($_REQUEST['date_end_month'   ]) : '' ;
          $end_day     = isset($_REQUEST['date_end_day']) ? _cloud_metering_filter_input($_REQUEST['date_end_day'   ])     : '' ;
        
          $start_date  = $start_year . '-' . $start_month . '-' . $start_day ;    
          $end_date    = $end_year . '-' . $end_month . '-' . $end_day ;
          
        }
        
        if (!empty($start_date)) {
          $start_date = date('c', strtotime($start_date));
        }
        
        if (!empty($end_date)) {
          $end_date = date('c', strtotime($end_date));
        }
        
        break;
      
      case 'Apply':
        
        $filter_column = isset($_REQUEST['operation']) ? _cloud_metering_filter_input($_REQUEST['operation']) : '';
        $filter_value  = isset($_REQUEST['filter'])    ? _cloud_metering_filter_input($_REQUEST['filter'])    : '';
        
        break;
      
      case 'Print PDF':
        
        break;
      
    }    
    
  }
  else {
    $filters['op'] = '';
  }

  $filters['date_filters' ]  = array('start_date' => $start_date, 'end_date' => $end_date);
  $filters['filter_column']  = $filter_column;
  $filters['filter_value' ]  = $filter_value;
  
  return $filters;
  
}

/**
 *  Get the date range for the bar graph
 */
function _cloud_dashboard_get_bar_date_range($start_date, $end_date, $period = 'daily') {
  
  $start_date = date_parse($start_date);
  $end_date = date_parse($end_date);
  
  $range = array();
  
  switch ($period) {
    
    case 'monthly':
      // Return each month
      
      // Change the dates to beginning of months
      $start_date = $start_date['year'] . '-' . $start_date['month'] . '-1 00:00:00';
      $start_date = date('c', strtotime($start_date));
      
      $end_date = $end_date['year'] . '-' . $end_date['month'] . '-1 00:00:00';
      $end_date = date('c', strtotime($end_date));
      
      $next_day = $start_date;
      while (true) {
        $date     = date_parse($next_day);
        $range[]  = array('month' => $date['month'], 'year' => $date['year']);
        if ($next_day == $end_date) {
          break;
        }
        $next_day = date('c', strtotime('+1 month', strtotime($next_day)));
      }
      
      break;
    
    case 'yearly':
      // Return each year
      
      // Change the dates to beginning of years
      $start_date = $start_date['year'] . '-01-01 00:00:00';
      $start_date = date('c', strtotime($start_date));
      
      $end_date = $end_date['year'] . '-01-01 00:00:00';
      $end_date = date('c', strtotime($end_date));
      
      $next_day = $start_date;
      while (true) {
        $date     = date_parse($next_day);
        $range[]  = array('year' => $date['year']);
        if ($next_day == $end_date) {
          break;
        }
        $next_day = date('c', strtotime('+1 year', strtotime($next_day)));
      }
      
      break;
    
    default:
      // Return each day
      
      // Change the dates to beginning of the day
      $start_date = $start_date['year'] . '-' . $start_date['month'] . '-' . $start_date['day'] . ' 00:00:00';
      $start_date = date('c', strtotime($start_date));
      
      $end_date = $end_date['year'] . '-' . $end_date['month'] . '-' . $end_date['day'] . ' 00:00:00';
      $end_date = date('c', strtotime($end_date));
      
      $next_day = $start_date;
      while (true) {
        $date     = date_parse($next_day);
        $range[]  = array('day' => $date['day'], 'month' => $date['month'], 'year' => $date['year']);
        if ($next_day == $end_date) {
          break;
        }
        $next_day = date('c', strtotime('+1 day', strtotime($next_day)));
      }
      
      break;
    
  }
  
  return $range;
}

/**
 * Function to find the difference between two dates
 */
function _cloud_dashboard_date_diff($d1, $d2) {

  $d1 = (is_string($d1) ? strtotime($d1) : $d1);
  $d2 = (is_string($d2) ? strtotime($d2) : $d2);

  $diff_secs = abs($d1 - $d2);
  $base_year = min(date("Y", $d1), date("Y", $d2));

  $diff = mktime(0, 0, $diff_secs, 1, 1, $base_year);
  return array(
    "years" => date("Y", $diff) - $base_year,
    "months_total" => (date("Y", $diff) - $base_year) * 12 + date("n", $diff) - 1,
    "months" => date("n", $diff) - 1,
    "days_total" => floor($diff_secs / (3600 * 24)),
    "days" => date("j", $diff) - 1,
    "hours_total" => floor($diff_secs / 3600),
    "hours" => date("G", $diff),
    "minutes_total" => floor($diff_secs / 60),
    "minutes" => (int) date("i", $diff),
    "seconds_total" => $diff_secs,
    "seconds" => (int) date("s", $diff)
  );
  
} 
