<?php

/**
 * @file
 * Provides PDF generation functionalites for cloud_dashboard.module.
 * Works with Cloud Metering Module
 * 
 * Copyright (c) 2011 DOCOMO Innovations, Inc.
 *
 * 
 */

//memory settings  
ini_set( 'memory_limit', '100M' );

$configs = _cloud_dashboard_tcpdf_config();
  
if ($configs === FALSE) {
  return;
}
  
$tcpdf_path = $configs['tcpdf_path'];
  
require_once $tcpdf_path . '/config/lang/eng.php';
require_once $tcpdf_path . '/tcpdf.php';
require_once $tcpdf_path . '/htmlcolors.php';

// Extend the TCPDF class to create custom Header and Footer
class MYPDF extends TCPDF {

  //Page header
  public function Header() {
    //Get the actual file path of the Logo image
    $get_tcpdf_logo = realpath($_SERVER['DOCUMENT_ROOT'] . base_path() . variable_get('tcpdf_logo', ''));

    //Check the image type
    $file_info = getimagesize($get_tcpdf_logo);
    $mime_type = $file_info['mime'];
    
    switch ($mime_type) {
       case "image/jpeg":   
        // Logo      
        if (!empty($get_tcpdf_logo)) {
         $image_file = $get_tcpdf_logo;
         $this->Image($image_file, 20, 10, 35, '', 'JPG', '', 'T', FALSE, 300, '', FALSE, FALSE, 0, FALSE, FALSE, FALSE);
        }
      break;
      
      case "image/gif":
        // Logo      
        if (!empty($get_tcpdf_logo)) {
         $image_file = $get_tcpdf_logo;
         $this->Image($image_file, 20, 10, 35, '', 'GIF', '', 'T', FALSE, 300, '', FALSE, FALSE, 0, FALSE, FALSE, FALSE);
        }
      break;
    }

    
    // Set font
    $this->SetFont('helvetica', 'B', 12);
    // Title
    $this->SetXY(15, 10);
    $this->Cell(0, 15, 'Cloud Instances Report', 0, FALSE, 'C', 0, '', 0, FALSE, 'M', 'M');
  }

  // Page footer
  public function Footer() {
    // Position at 15 mm from bottom
    $this->SetY(-15);
    // Set font
    $this->SetFont('helvetica', 'I', 8);
    // Page number
    $this->Cell(0, 10, 'Page '. $this->getAliasNumPage() . '/' . $this->getAliasNbPages(), 0, FALSE, 'C', 0, '', 0, FALSE, 'T', 'M');
  }
}

function _cloud_dashboard_generate_pdf_report($html_data) {
  // create new PDF document
  $pdf = new MYPDF(PDF_PAGE_ORIENTATION, PDF_UNIT, 'A4', TRUE, 'UTF-8', FALSE);

  // set document information
  $pdf->SetCreator(PDF_CREATOR);
  /*
  $pdf->SetAuthor('Nicola Asuni');
  $pdf->SetTitle('TCPDF Example 006');
  $pdf->SetSubject('TCPDF Tutorial');
  $pdf->SetKeywords('TCPDF, PDF, example, test, guide');
  */

  // set default header data
  $pdf->SetHeaderData(PDF_HEADER_LOGO, PDF_HEADER_LOGO_WIDTH, PDF_HEADER_TITLE . ' 006', PDF_HEADER_STRING);

  // set header and footer fonts
  $pdf->setHeaderFont(Array(PDF_FONT_NAME_MAIN, '', PDF_FONT_SIZE_MAIN));
  $pdf->setFooterFont(Array(PDF_FONT_NAME_DATA, '', PDF_FONT_SIZE_DATA));

  // set default monospaced font
  $pdf->SetDefaultMonospacedFont(PDF_FONT_MONOSPACED);

  //set margins
  //$pdf->SetMargins(PDF_MARGIN_LEFT, PDF_MARGIN_TOP, PDF_MARGIN_RIGHT);

  $pdf->SetMargins(20, PDF_MARGIN_TOP, 10);
  $pdf->SetHeaderMargin(PDF_MARGIN_HEADER);
  $pdf->SetFooterMargin(PDF_MARGIN_FOOTER);

  //set auto page breaks
  $pdf->SetAutoPageBreak(TRUE, PDF_MARGIN_BOTTOM);

  //set image scale factor
  $pdf->setImageScale(PDF_IMAGE_SCALE_RATIO);

  //set some language-dependent strings
  $pdf->setLanguageArray($l);

  // ---------------------------------------------------------

  // set font
  $pdf->SetFont('dejavusans', '', 10);

  // add a page
  $pdf->AddPage();

  // Add the HTML content
  $html .= $html_data;
  $html .= '<p style="font-size: 22px;">Report Generated on: ' . date('Y-m-d') . ' at ' . date('h:i:s A T P') . '</p>';

  //Set the XY position
  $pdf->SetXY(15, 25);

  // output the HTML content
  $pdf->writeHTML($html, TRUE, FALSE, TRUE, FALSE, '');

  //Today's Date
  $date_today = date('Y-m-d H-m-s');

  //Close and output PDF document
  $pdf->Output('Cloud_Dashboard_' . $date_today . '.pdf', 'D');
  ob_end_flush();
}

/**
 * Function to generate PDF report of cloud instances
 */

function _cloud_dashboard_generate_pdf_data($arg, $page_argument_display) {
  //Global $user object;
  global $user;
  
  $pdf_configs = _cloud_dashboard_tcpdf_config();  
  
  if ($pdf_configs === FALSE) {
    drupal_set_message(t('TCPDF library is required for generating PDF report. Please go to <i>' . l('Cloud Dashboard Settings', 'admin/settings/cloud_dashboard') . '</i> Page and configure the TCPDF settings.'));
    return TRUE;
  }
  
  //Get the page arguments
  $page_argument = filter_xss($arg);
  if (empty($page_argument)) {
    //Argument for PDF generation of All clouds are selected
    $page_argument = 'all';
  }
    
  $filter     = isset($_REQUEST['filter'   ]) ? $_REQUEST['filter'   ] : '' ;
  $filter_col = isset($_REQUEST['operation']) ? $_REQUEST['operation'] : 0  ; // default: Nickname
  $filter     = trim($filter);
  
  $order_type     = isset($_REQUEST['order'   ]) ? $_REQUEST['order'   ] : '' ;
  $order_by       = isset($_REQUEST['sort'   ]) ? $_REQUEST['sort'   ] : '' ;
  
  if (module_exists('date_popup')) {
    $filter_start_date = isset($_REQUEST['date_start'   ]) ? $_REQUEST['date_start'   ] : '' ;
    $filter_end_date = isset($_REQUEST['date_end'   ]) ? $_REQUEST['date_end'   ] : '' ;
  }
  else {
    $filter_start_date['year'] = isset($_REQUEST['date_start_year']) ? $_REQUEST['date_start_year'   ] : '' ;
    $filter_start_date['month'] = isset($_REQUEST['date_start_month']) ? $_REQUEST['date_start_month'   ] : '' ;
    $filter_start_date['day'] = isset($_REQUEST['date_start_day']) ? $_REQUEST['date_start_day'   ] : '' ;
      
    $filter_end_date['year'] = isset($_REQUEST['date_end_year']) ? $_REQUEST['date_end_year'   ] : '' ;
    $filter_end_date['month'] = isset($_REQUEST['date_end_month']) ? $_REQUEST['date_end_month'   ] : '' ;
    $filter_end_date['day'] = isset($_REQUEST['date_end_day']) ? $_REQUEST['date_end_day'   ] : '' ;
  }
  
    
  if ( $filter_col == 0 )  {
    $column  = 'Nickname'           ;
    $sql_col = 'instance_nickname'  ;
  }
  elseif ($filter_col == 1 ) {
    $column  = 'ID'                 ;
    $sql_col = 'instance_id'        ;
  }
  elseif ($filter_col == 2 ) {
    $column  = 'Public-DNS-Name'    ;
    $sql_col = 'dns_name'           ;
  }
    
  if ( isset($filter) ) {
    
    $query_args[] = $sql_col ;
    $query_args[] = $filter  ;
  }
  else {
    $filter       = ' 1 ' ;
    $query_args[] = ' ' ;
  }
  
  //Check Date PopUp module exists
  if (module_exists('date_popup')) {
    //Convert the Date into the Y-m-d format
    $get_start_date_filter = strtotime($filter_start_date['date']);
    $get_start_date = date('c', $get_start_date_filter);
  
    $get_end_date_filter = strtotime($filter_end_date['date'] . '23:59:59');
    $get_end_date = date('c', $get_end_date_filter);
  }
  else {
    //Convert the Date into the Y-m-d format
    $get_start_date_filter = strtotime($filter_start_date['year'] . '-' . $filter_start_date['month'] . '-' . $filter_start_date['day'] );
    $get_start_date = date('c', $get_start_date_filter);
  
    $get_end_date_filter = strtotime($filter_end_date['year'] . '-' . $filter_end_date['month'] . '-' . $filter_end_date['day'] . '23:59:59');
    $get_end_date = date('c', $get_end_date_filter);
  }
  
  //$cloud_list_data = array();
  
  //Get the list of all Cloud instances
  $cloud_list = cloud_get_all_clouds();
  
  if ($page_argument != 'all') {
    $cloud_list_data[$page_argument] = $page_argument;
  }
  else {
    $cloud_list_data = $cloud_list;
  }
    
  $filter_params = array();
  
  //Get the names of the Cloud instances
  $cloud_name_list = cloud_get_all_clouds_display_name();
  $count_clouds = 0;
  $filter_params = array( 'column' => $sql_col, 'filter_value' => $filter ) ;
  
  $total_cost_disp = 0;
  $rows = array();
  
  $all_instances_list = array();
  //Get all the instances w.r.t to Cloud Type
  foreach ($cloud_list_data as $cloud_context) {
    
    $instances_list_data = array();    
    $instances_list = _cloud_dashboard_generate_instance_data($cloud_context);
     
    if (is_array($instances_list) ) {
      //Validate the Instances for empty Instances and Terminated Instances
      //Add into Instance array the Instance Terminated date
      //Add into Instance array the Instance runtime cost
      $instances_list_data = _cloud_dashboard_get_instances_details($instances_list); 
    }
    else {      
      $instances_list_data = $instances_list;
    }    
    $all_instances_list = array_merge( $all_instances_list , $instances_list_data ) ;
  }
  
  //Construct the table header
  foreach ($cloud_list_data as $cloud_context) {
    if ($cloud_context == $page_argument) {
      $pdf_display_name = $cloud_name_list[$cloud_context];            
      $output_cloud .= '<tr>';
      $output_cloud .= '<td colspan="4" style="border-bottom: 1px solid #CC0033; text-align: center; font-weight: bold;">' . $pdf_display_name . '</td></tr>'; 
        
      if (!empty($page_argument) && $page_argument != 'all') {
        $cloud_name = $page_argument;
        $cloud_instance_data = cloud_get_instances($cloud_name, $filter_params);
      }          
    } 
    else {
      $cloud_instance_data = cloud_get_instances($cloud_context, $filter_params);
      $pdf_display_name = 'All'; 
      if ($count_clouds == 0 && $page_argument == 'all') {
        $output_cloud .= '<tr>';
        $output_cloud .= '<td colspan="4" style="border-bottom: 1px solid #CC0033; text-align: center; font-weight: bold;">' . t('All Clouds') . '</td></tr>';
      }
      $count_clouds++;
    } 
  }
  
  //Separate all running and Terminated Instances
  $all_running_instances = array();
  $all_terminated_instances = array();
  $running_instances_data = array();
  $terminated_instances_data = array();
  
  foreach ($all_instances_list as $key => $value) {     
     if (empty($value['terminated_date'])) {
       $all_running_instances[$key] = $value;
     }
     elseif (!empty($value['terminated_date'])) {
       $all_terminated_instances[$key] = $value;
     } 
  }      
  
  
  //Create the table for display
  $running_instances_data    = _cloud_dashboard_sorting_pdf_report($all_running_instances);
  $terminated_instances_data = _cloud_dashboard_sorting_pdf_report($all_terminated_instances);

  //Total cost
  $total_cost_disp = $running_instances_data['total_cost'] + $terminated_instances_data['total_cost'];
    
  $output = '<table>';
  $output .= '<tr><td width="15%">Total Cost</td><td width="20%">:  $' . number_format($total_cost_disp, 2) . '</td></tr>';
  $output .= '<tr><td width="15%"></td><td width="20%"></td></tr>';
  $output .= '</table>';
  
  //Add the Instances Listings
  $output .= '<table style="border: 1px solid #CC0033;">';
  $output .= $output_cloud;
  $output .= '<tr style="font-weight: bold"><td width="25%" style="border-right: 1px dotted #CC0033; border-bottom: 1px dotted #CC0033;">Nickname</td><td width="15%" style="border-right: 1px dotted #CC0033; border-bottom: 1px dotted #CC0033;">ID</td><td width="20%" style="border-right: 1px dotted #CC0033; border-bottom: 1px dotted #CC0033;">SSH Key</td><td width="25%" style="border-right: 1px dotted #CC0033; border-bottom: 1px dotted #CC0033;">Host Name</td><td width="15%" style="border-bottom: 1px dotted #CC0033; text-align: right;">Cost</td></tr>';
  $output .= $running_instances_data['pdf_data'];
  $output .= $terminated_instances_data['pdf_data'];
  $output .= '</table>';
    
  //Generate the PDF Report
  _cloud_dashboard_generate_pdf_report($output); 
   
}


function _cloud_dashboard_sorting_pdf_report($all_instances_list) {
  $output_array = array();
  $total_cost_calc = 0;
  
  foreach ($all_instances_list as $key => $get_list_data) {
    $row = array();

    //Get the Instance Object details from Cloud API
    $instance_data = cloud_get_instance($get_list_data['cloud_context'], $key);
            
    //Get the cloud type, instance id and instance_name
    $get_id =  $get_list_data['instance_id'];
    $get_context = $get_list_data['cloud_context'];
    $get_instance_name = $get_list_data['instance_nickname']; 
    $get_key_name = $get_list_data['key_name'];
    $hostname = $get_list_data['hostname'];
    $get_instance_status = $get_list_data['instance_state_name'];
            
    $public_dns = substr( $hostname, 0, strpos($hostname,  '.') ) ;

    if ( strlen($public_dns) > 3 ) { // truncate only in case the name is long      
      $public_dns = substr($hostname, 0, strpos($hostname,  '.')) . '...' ;
    }
    else {
      $public_dns = $hostname ;
    }         

    $get_instance_name = !empty($get_instance_name) ? ($get_instance_name) : $get_id;
    $public_dns = !empty($public_dns) ? ($public_dns) : 'Instance ' . ucwords($get_instance_status);
    $get_key_name = !empty($get_key_name) ? ($get_key_name) : '-- NA --';
            
    $total_instance_type_charge = $get_list_data['total_cost'];
    $total_cost = t( '$' . number_format($total_instance_type_charge, 2));

    //Construct the data rows to displaying and sorting
    $row['name'] = $get_instance_name;
    $row['instance_id'] = $get_id;
    $row['key_name'] = $get_key_name;
    $row['dns'] = $public_dns;
    $row['cost'] = $total_cost;
              
    $rows[$get_id] = $row;
            
    //Check the Sorting of the Cloud Dashboard listing by type
    if (!empty($order_type)) {
      if ($order_type == 'ID') {
        $sort_by_key[$get_id] = $get_list_data['instance_id'];
      }
      elseif ($order_type == 'Nickname') {
        $sort_by_key[$get_id] = $row_data['name'];
      }
      elseif ($order_type == 'Hostname') {
        $sort_by_key[$get_id] = $public_dns;
      }
      else {
        $sort_by_key[$get_id] = $total_instance_type_charge;
      }
    }
    else {
      $sort_by_key[$get_id] = $total_instance_type_charge;
    } 
    
    $total_cost_disp += $total_instance_type_charge;          
    
  }
  
  //Sort the table as per Asecenind/Dsecending order
  if (!empty($order_by)) {
    if ($order_by == 'desc') {
      array_multisort($sort_by_key, SORT_DESC , $rows);
    }
    else {
      array_multisort($sort_by_key, SORT_ASC , $rows);
    }
  }
  else {
    array_multisort($sort_by_key, SORT_ASC , $rows);
  }
  
  foreach ($rows as $key => $value) {
    
    $output_data .= '<tr>';          
    $output_data .= '<td width="25%" style="border-right: 1px dotted #CC0033; border-bottom: 1px dotted #CC0033;">' . $value['name'] . '</td>';
    $output_data .= '<td width="15%" style="border-right: 1px dotted #CC0033; border-bottom: 1px dotted #CC0033;">' . $value['instance_id'] . '</td>';
    $output_data .= '<td width="20%" style="border-right: 1px dotted #CC0033; border-bottom: 1px dotted #CC0033;">' . $value['key_name'] . '</td>';
    $output_data .= '<td width="25%" style="border-right: 1px dotted #CC0033; border-bottom: 1px dotted #CC0033;">' . $value['dns'] . '</td>';
    $output_data .= '<td width="15%" style="border-bottom: 1px dotted #CC0033; text-align: right;">' . $value['cost'] . '</td>';
    $output_data .= '</tr>';
  
  }
  
  $output_array['pdf_data']   = $output_data;
  $output_array['total_cost'] = $total_cost_disp; 
  
  return $output_array;
}
