<?php

/**
 * @file
 * Contains private and non-hook methods used by Cloud Metering module
 * Works with Cloud Metering module
 * 
 * Copyright (c) 2011 DOCOMO Innovations, Inc.
 * 
 */

/**
 *  Configuration form
 */
function cloud_metering_configure() {
  
  // Get the filter form
  // $form = drupal_get_form('cloud_metering_configure_form');
  $form = drupal_get_form('cloud_metering_instances_list');
  
  return $form ;

}

/**
 *  Configuration Group users form
 */
function cloud_metering_user_configure() {
  // We get the Drupal Role
  // Display all the users for the Role
  // Where threshold can be set against each User and the Role
    
  // Get the filter form
  $form = drupal_get_form('cloud_metering_user_list');
  
  //$table = theme('cloud_metering_user_list');
    
  return $form;  
}

/**
 *  Theme the cloud instances listing
 */
function theme_cloud_metering_instances_list($form) {

  $clouds = cloud_get_all_clouds();
  $all_instances = array();
  
  $selected_cloud = $form['options']['cloud']['#value'];
  
  if (empty($selected_cloud)) {
    foreach ($clouds as $cloud) {
      $instances = cloud_get_instances($cloud);
      if (is_array($instances)) {
        $all_instances = array_merge($all_instances, $instances); 
      }
    }
  }
  else {
    $all_instances = cloud_get_instances($selected_cloud);
  }
  
  if (!is_array($all_instances)) {
    $all_instances = array();
  }
  
  $headers = array(
    t('Name'),
    t('Id'),
    t('Host Name'),
    t('State'),
    t('Budget ($)'),
    t('Threshold (%)'),
    t('Operations'),
  );
  
  $rows = array();
  
  ksort($all_instances);
  
  // Custom pagination
  $page  = (isset($_REQUEST['page']) ? $_REQUEST['page'] : 0);
  $page  = (is_numeric($page) ? $page : 0);
  $count = sizeof($all_instances);
  $start = $page * CLOUD_PAGER_LIMIT;
  $max   = (( $page + 1 ) * CLOUD_PAGER_LIMIT < $count ? ( $page + 1 ) * CLOUD_PAGER_LIMIT : $count);
  $i     = 0;
  
  foreach ($all_instances as $instance) {
    
    if ($i < $start) {
      $i++;
      continue;
    }
    
    $budget_data = _cloud_metering_get_instance_budget($instance['cloud_context'], $instance['instance_id']);
    $budget      = $budget_data['budget'];
    $threshold   = $budget_data['threshold'];
    
    $hostname = $instance['hostname'];
    $public_dns = substr($hostname, 0, strpos($hostname,  '.')) ;
    if (strlen($public_dns) > 3) {
      $public_dns = substr($hostname, 0, strpos($hostname,  '.')) . '...' ;
    }
    else {
      $public_dns = $hostname ;
    }
    
    if (!empty($public_dns)) {
      $public_dns = l($public_dns, 'http://' . $hostname, array('attributes' => array('target' => '_blank')));
    }
    
    $cloud_context   = $instance['cloud_context'];
    $instance_id     = $instance['instance_id'];

    $lock_status     = cloud_perform_action('', 'get_instance_lock_status', $cloud_context, array('instance_id' => $instance_id));
    $instance_locked = ( $lock_status == 'Lock' ? TRUE : FALSE );
    
    $cloud_menu_path = cloud_get_menu_path($cloud_context);
    
    if ($instance_locked) {
      $link = l(t('Instance locked'), $cloud_menu_path . '/instances/describe/' . $instance_id . '/info', array('query' => 'instance_id=' . urlencode($instance_id), 'fragment' => 'edit-lock-text'));
    }
    else {
      $link = l(t('Edit Budget'), 'admin/settings/cloud_metering/threshold/' . $cloud_context . '/' . $instance_id, array('query' => drupal_get_destination()));
    }
    
    $nickname   = l($instance['name'], $cloud_menu_path . '/instances/describe/' . $instance_id . '/info', array('query' => 'instance_id=' . urlencode($instance_id)));
    
    $rows[] = array(
      $nickname,
      $instance['instance_id'],
      $public_dns,
      ucwords($instance['state']),
      '<div style="text-align: right;">' . number_format($budget, 2, '.', ',') . '</div>',
      '<div style="text-align: right;">' . $threshold . '</div>',
      $link,
    );
    
    $i++;
    
    if ($i >= $max) {
      break;
    }
  }
  
  $table = theme('table', $headers, $rows);
  
  global $pager_page_array, $pager_total, $pager_total_items;
  
  $pager_total_items[0] = $count;
  $pager_total[0]       = ceil($count / CLOUD_PAGER_LIMIT);
  $pager_page_array[0]  = $page;
  
  $pager = theme('pager', NULL, CLOUD_PAGER_LIMIT);
  
  $form['instances_list'] = array(
    '#type' => 'markup',
    '#value' => $table . $pager,
  );
  
  return drupal_render($form);
}

/**
 *  Configuration form
 */
function cloud_metering_instances_list(&$form_state) {
  
  $selected_cloud = arg(3);
  if (!empty($selected_cloud)) {
    $selected_cloud = _cloud_metering_filter_input($selected_cloud);
  }
  
  $form = array();
  
  $form['options'] = array(
    '#type' => 'fieldset',
    '#prefix' => '<div class="container-inline">',
    '#suffix' => '</div>'
  );
  
  // Get all clouds
  $clouds = cloud_get_all_clouds();
  
  $options = array('' => t(' - Any - '));
  //Get the names of the Cloud instances
  $cloud_name_list = cloud_get_all_clouds_display_name();
  
  foreach ($clouds as $cloud) {
    $options[$cloud] = $cloud_name_list[$cloud];
  }
  
  $form['options']['label'    ] = array('#type' => 'item'     , '#title'   => t('Cloud'),                                      ) ;
  $form['options']['clouds'   ] = array('#type' => 'select'   , '#options' => $options,   '#default_value' => $selected_cloud, ) ;
  $form['options']['submit'   ] = array('#type' => 'submit'   , '#value'   => t('Apply'),                                      ) ;
  $form['options']['cloud'    ] = array('#type' => 'hidden'   , '#value'   => $selected_cloud                                  ) ;
  
  return $form;
}

/**
 *  Submit handler for configuration form
 */
function cloud_metering_instances_list_submit($form, &$form_state) {
  
  if (!empty($form_state['values']['clouds'])) {
    drupal_goto('admin/settings/cloud_metering/' . $form_state['values']['clouds']);
  }
  else {
    drupal_goto('admin/settings/cloud_metering');
  }
  
}

/**
 * Add/Edit Group and Users for Initial Budget
 */
function cloud_metering_user_list($form_state) {
  global $user;
  
  $rid       = filter_xss(arg(4));
  $operation = filter_xss(arg(5));
  
  if (!empty($rid)) {
    $arg = array('column' => 'rid', 'value' => $rid);
  }
  else {
    $arg = '';
  }
  
  $get_group_details = array(
    'name' => '',
    'user_data' => '',
    'grid' => '',
    'group_budget' => '',
    'gid' => '',
  );
  
  if (!empty($arg)) {
    $group_details = _cloud_metering_get_group_details($arg);
    $get_group_details = $group_details[$rid];
  }
    
  if (!user_access('configure user budget')) {   
    drupal_set_message('You do not have permission to Edit the Group Budget or Group ID is invalid!');
    drupal_goto('admin/settings/cloud_metering');
  }
  elseif (!empty($gid) && empty($get_group_details)) {
    drupal_set_message('You do not have permission to Edit the Group Budget or Group ID is invalid!');
    drupal_goto('admin/settings/cloud_metering');
  }
  
  $form = array();
  
  $form['wrapper'] = array(
     '#type' => 'fieldset',
     '#title' => '',
     '#weight' => -1
  );
   
  $form['wrapper']['group_name'] = array(
     '#type' => 'item',
     '#title' => t('Group Name'),
     '#value' => $get_group_details['name']
  );
  
  $arg = array('column' => 'grid', 'value' => $rid);
  $group_budget = _cloud_metering_get_group_budget($arg);
  
  if (!empty($group_budget)) {
    $group_budget_value = $group_budget[$rid]['group_budget'];  
    $admin_users = unserialize($group_budget[$rid]['user_data']);
    $group_threshold_value =  $group_budget[$rid]['threshold'];
  }
  else {
    $group_budget_value = 0;  
    $admin_users = '';
    $group_threshold_value = 0;
  }
  
    
  if (empty($admin_users)) {
    $admin_users = array();
  }
  
  $form['wrapper']['group_budget'] = array(
    '#type' => 'textfield',
    '#title' => t('Group Budget ($)'),
    '#default_value' => $group_budget_value,
    '#description' => t('Enter the Group Budget Value. Put only the numeric value without a currency symbol.'),
    '#required' => TRUE,
    '#size' => 20
  );
  
  
  $form['wrapper']['group_threshold'] = array(
    '#type' => 'textfield',
    '#title' => t('Group Threshold (%)'),
    '#default_value' => $group_threshold_value,
    '#description' => t('Enter the Group Threshold limit. Put only the numeric value without a percentage symbol.'),
    '#required' => TRUE,
    '#size' => 20
  );
  
  //Get all the users for specific role  
  $get_all_users = _cloud_metering_get_users_list_by_role($rid);
  
  $options = array();
  $val = array();
  
  foreach ($get_all_users as $key => $value) {
    
    $user_budget_amount = _cloud_metering_get_user_budget($key);
    
    $form[$key]['name']=array('#value' => $value['name']);
    $form[$key]['mail']=array('#value' => $value['mail']);
    $form[$key]['user_budget_' . $key] = array('#type' => 'textfield', '#default_value' => $user_budget_amount, '#size' => '20');
    $form[$key]['user_budget_value_' . $key] = array('#type' => 'hidden', '#value' => $user_budget_amount);
    //Construct the select boxes values
    $val[$key] = '';
  }
  
  $form['process_user'] = array(
    '#type' => 'checkboxes',
    '#options' => $val,
    '#default_value' => $admin_users, 
  );
  
  $form['group_id'] = array(
    '#type' => 'hidden',
    '#value' => $rid
  );
  
  $form['group_budget_value'] = array(
    '#type' => 'hidden',
    '#value' => $group_budget_value,
  );
    
  $form['submit'] = array(
   '#type' => 'submit',
   '#value' => 'Update',
   '#submit' => array('_cloud_metering_user_list_submit'),
   '#validate' => array('_cloud_metering_user_list_validate'),
   '#weight' => 2
  );
   
  return $form;
}


/**
 *  Theme the user listing
 */
function theme_cloud_metering_user_list($form) {
  
  $rows = array();
  
  foreach (element_children($form) as $key) {
      $row = array();
            
      if (isset($form[$key]['mail'])) {
        $status = drupal_render($form['process_user'][$key]);
        $row[] = array('data' => $status);
        $row[] = array('data' => drupal_render($form[$key]['name'])); 
        $row[] = array('data' => drupal_render($form[$key]['mail']));
        $row[] = array('data' => drupal_render($form[$key]['user_budget_' . $key]));   
        $rows[] = $row;
            
      }
  }
  
  // Individual table headers.
  $header = array();
  
  $header[] = t('Group Admin');   
  $header[] = t('Name');
  $header[] = t('Email');
  $header[] = t('User Budget ($)');
  
  $search_result = theme('table', $header, $rows);
  $form['search_result']['#value'] = $search_result;
  
  $output = drupal_render($form);
  
  
  return $output;
}

/**
 *  User Budget configuration
 */
function cloud_metering_user_budget_configure() {
  global $user;
  $group_arg = filter_xss(arg(4));
  
  $class_all = '';
  
  if (empty($group_arg) || $group_arg == 'all') {
    $class_all = 'active';
    $get_group_id = '';
  }
  else {
    $get_group_id = $group_arg;
  }
  
  if (user_access('configure user budget')) {
    if ($user->uid > 1) {
      $arg = array('column' => 'guid', 'value' => $user->uid);
      $user_details = _cloud_metering_get_group_details($arg);
      
      if ($group_arg == 'user') {
         $output .= drupal_get_form('_cloud_metering_add_users_to_group');
         return $output;
      }
      
      $user_group_role = $user_details[$user->uid]['grid'];
      $user_group_id = $user_details[$user->uid]['gid'];
    
      if (empty($user_details) || $user_group_id != $get_group_id) {
        drupal_set_message('You do not have permission to View this User Group');
        drupal_goto(drupal_get_path_alias('user/'. $user->uid));
        return; 
      }
      
      $arg = array('column' => 'gid', 'value' => $get_group_id);
      $group_user_details = _cloud_metering_get_group_users_details($arg);
    
      $query_user = db_query('SELECT a.* FROM { users } a INNER JOIN { users_roles } b WHERE a.uid = b.uid and b.rid = %d', $user_group_role);    
      
    }
    else {
    
      $arg = array('column' => 'gid', 'value' => $get_group_id);
      $group_user_details = _cloud_metering_get_group_users_details($arg);
          
      $query_user = db_query('SELECT a.* FROM { users } a INNER JOIN { users_roles } b WHERE a.uid = b.uid and a.uid > 1');
    }
  }
  else {
    drupal_set_message('You do not have permission to Edit User Budget');
    drupal_goto(drupal_get_path_alias('user/'. $user->uid));
    return; 
  }
  
  $group_details = _cloud_metering_get_group_details();
  
  $output_tab = '<div id="tabs-wrapper"><ul style="float: none;" class="tabs primary">';
  $output_tab .= '<li style="display: inline;" class="' . $class_all . '">' . l(t('All Groups'), 'admin/settings/cloud_metering/group_users/all', array('attributes' => array('class' => $class_all)) ) . '</li>';
  
  foreach ($group_details as $key => $values) {
    $group_name = $values['name'];
    $group_id = $values['gid'];
    
    $class_name = '';
    if ($group_arg == $group_id) {
      $class_name = 'active';
    }
    
    $output_tab .= '<li style="display: inline;" class="' . $class_name . '">' . l( $group_name, 'admin/settings/cloud_metering/group_users/' . $group_id, array('attributes' => array('class' => $class_name)) ) . '</li>';
  }
  
  $output_tab .= '</ul></div>';
  
  $headers = array(
    //t('User Id'),
    t('User Name'),
    t('E-Mail'),
    t('Created'),
    t('Last Login'),
    t('User Budget ($)'),
    t('')
  );
  
  $rows = array();
  
  foreach ($group_user_details as $r) {
    $row = array();
    $gu = user_load($r['uid']);
    $row[] = $gu->name;
    $row[] = str_replace('@', '&#64;', $gu->mail);
    $row[] = date('Y-m-d H:i:s', $gu->created);
    $row[] = date('Y-m-d H:i:s', $gu->access);
    $row[] = number_format(_cloud_metering_get_user_budget($gu->uid), 2, '.', ',');
   
    if (user_access('configure user budget')) {
      $row[] = l('Edit', 'admin/settings/cloud_metering/group_users/user/' . $gu->uid . '/edit' );
    }
   else {
     $row[] = '';
   }
   $rows[] = $row;
   
  }
  $table = theme('table', $headers, $rows);
  
  $output = t('User Budget Settings');
  
  if ($group_arg == 'user') {
    $output .= drupal_get_form('_cloud_metering_add_users_to_group');
  }
  else {
    $location = 'window.location="' . base_path() . 'admin/settings/cloud_metering/group_users/user' . '"';
    $form['add'] = array(
      '#type' => 'button',
      '#value' => 'Add User to Group',
      '#attributes' => array('onclick' => $location),
    );
    
    $output .= $output_tab . $table . drupal_render($form);
  }
  
  return $output;
}

/**
 *  Group Budget configuration
 */
function cloud_metering_group_budget_configure() {
  
  $rid       = filter_xss(arg(4));
  $operation = filter_xss(arg(5));
  
  if (!empty($rid) && !empty($operation)) {
    return cloud_metering_user_configure();
  }
  
  $output = drupal_set_title(t('Group Budget Settings'));  
  $output .= _cloud_metering_get_all_groups();
  return $output;

}

/**
 * Validate the group details form
 */

function _cloud_metering_user_list_validate($form, &$form_state) {
  
  $values = $form_state['values'];
  
  $group_threshold = $values['group_threshold'];
  $group_budget = $values['group_budget'];
  $group_budget_value = $values['group_budget_value'];
  $group_id = $values['group_id'];
  
  $operation = $values['submit'];
    
  if (!is_numeric($group_budget)) {
    form_set_error('group_budget', t('Please enter only numeric values only.'));
  }
  elseif ($group_budget <= 0) {
    form_set_error('group_budget' , t('Please enter values greater than Zero(0)'));
  }
  
  if (!is_numeric($group_threshold)) {
    form_set_error('group_threshold', t('Please enter only numeric values only.'));
  }
  elseif ($group_threshold <= 0) {
    form_set_error('group_threshold' , t('Please enter values greater than Zero(0)'));
  }
  elseif ($group_threshold > 100) {
    form_set_error('group_threshold' , t('Group Threshold percentage cannot be more than 100%'));
  }
  
  
  if ($operation == 'Update') {
    
    $group_budget_org = db_result(db_query('SELECT group_budget FROM {' . CLOUD_METERING_BUDGET_GROUP_TABLE . '} WHERE grid = %d', $group_id));  
      
    $total_user_budget = 0;
  
    $group_user_details = _cloud_metering_get_users_list_by_role($group_id);
    
    $total_new_budget = 0;
    $total_user_budget = 0;
    foreach ($group_user_details as $key => $value) {
     
      if ($value['rid'] == $group_id) {
        $get_budget = _cloud_metering_get_user_budget($key);
      }
       
      $new_budget_value = $values['user_budget_' . $key];
      
      $total_user_budget += $get_budget;
      $total_new_budget += $new_budget_value;
    }
        
    if ($group_budget < $total_user_budget || $group_budget < $total_new_budget) {
      form_set_error('group_budget' , t('Group Budget cannot be less than the Total Budget allocated to the Users of the Group.'));
    }
  }
  
  return;
}

/**
 * Submit the new Group Details
 */
function _cloud_metering_user_list_submit($form, &$form_state) {
  
  $values = $form_state['values'];
  $group_id = $values['group_id'];
  $group_threshold = $values['group_threshold']; 
  $group_budget = $values['group_budget'];
  $group_budget_value = $values['group_budget_value'];
  
  $user_data = serialize($values['process_user']);
  
  //Get all users for the Group
  $get_all_users = _cloud_metering_get_users_list_by_role($group_id);
  
  $get_time  = strtotime('now');
  $operation = $values['submit'];
  
  if (!$group_budget_value) {
    $add_group = db_query('INSERT INTO {' . CLOUD_METERING_BUDGET_GROUP_TABLE . "} (gid, user_data, grid, threshold, group_budget) VALUES(NULL, '%s', %d, %d, %f)",  $user_data, $group_id, $group_threshold, $group_budget);
    
    if ($add_group) {
      //Add the User Budget values
      foreach ($get_all_users as $key => $budget) {
        $user_budget = $values['user_budget_' . $key];
        $add_user = db_query('INSERT INTO {' . CLOUD_METERING_USER_BUDGET_TABLE . '} (id, uid, grid, threshold, budget) VALUES(NULL, %d, %d, %d, %f)',  $key, $group_id, $group_threshold, $user_budget);
      }
      drupal_set_message(t('Group budget settings saved.'));
      drupal_goto('admin/settings/cloud_metering/group_settings');
    }
  }
  else {
    
    $update_group = db_query('UPDATE {' . CLOUD_METERING_BUDGET_GROUP_TABLE . "} SET user_data = '%s', threshold = %d, group_budget = %f WHERE grid = %d", $user_data, $group_threshold, $group_budget, $group_id);
    
    if ($update_group) {
      //Add the User Budget values
      foreach ($get_all_users as $key => $budget) {
        $user_budget = $values['user_budget_' . $key];
        $add_user = db_query('UPDATE {' . CLOUD_METERING_USER_BUDGET_TABLE . '} SET budget = %f WHERE uid = %d and grid = %d',  $user_budget, $key, $group_id);
      }
      drupal_set_message(check_plain('Group details are updated.'));
      drupal_goto('admin/settings/cloud_metering/group_settings');
    }
    
  }
}


/**
 * Function validating User and Group Budget changes
 */
function _cloud_metering_check_budget_changes($group_id, $instances_budget = '', $user_id, $budget) {
  
  $invalid = 0;
  $current_budget = _cloud_metering_get_user_budget($user_id) ;
  
  $group_budget = db_result(db_query('SELECT group_budget FROM {' . CLOUD_METERING_BUDGET_GROUP_TABLE . '} WHERE gid = %d', $group_id));  
      
  $total_user_budget = 0;
  
  $arg = array('column' => 'gid', 'value' => $group_id);
  $group_user_details = _cloud_metering_get_group_users_details($arg);
 
  foreach ($group_user_details as $key => $value) {
    if ($value['gid'] == $group_id) {
      $get_budget = _cloud_metering_get_user_budget($key);
    }
    $total_user_budget += $get_budget;
  }
  
  if (!empty($instances_budget)) {
    $check_budget = ($instances_budget + $budget);
    $can_use_budget = ($current_budget - $check_budget);   
    $usable_budget = ($group_budget - $total_user_budget);
  } 
  else {
    $check_budget = ($total_user_budget - $current_budget);    
    $balance_budget = abs($group_budget - $total_user_budget);
    $can_use_budget = ($group_budget - $check_budget) - $budget;   
    $usable_budget = ($group_budget - $total_user_budget);
  }
    
  if ($budget > $group_budget) {
    $invalid = 1;
  }
  elseif ($can_use_budget < 0) {
    $invalid = 2;
  }
  
  return $invalid;
}

/**
 * Get Group Names
 */
function _cloud_metering_get_all_groups() {
   
  $headers = array(
    t('Group Role'      ),
    t('Group Budget ($)'),
    t('Threshold (%)'),
    t('Admin User'      ),
    t('Group Settings'  )
  );
  
  $rows = array();
  
  $groups = _cloud_metering_get_group_details();
  
  foreach ($groups as $g) {
    $row = array();
   
    $row[] = $g['name'];
   
    $arg = array('column' => 'grid', 'value' => $g['rid']);
    $group_budget = _cloud_metering_get_group_budget($arg);
   
    if (!empty($group_budget) && isset($group_budget[$g['rid']])) {
      $budget_value     = $group_budget[$g['rid']]['group_budget'];
      $threshold_value  = $group_budget[$g['rid']]['threshold'];
      $group_admin_data = unserialize($group_budget[$g['rid']]['user_data']);
    }
    else {
      $budget_value     = 0;
      $threshold_value  = 0;
      $group_admin_data = '';
    }
   
    
    $display_user_data = '';

    if (!empty($group_admin_data)) {
      foreach ($group_admin_data as $key => $user) {
        if ($user > 0) {
          $user_data = user_load($user);
          $display_user_data .= $user_data->mail . '<br/>';
        } 
      }
    }
   
    //Group Budget
    if ($budget_value) {
      $row[] = number_format($budget_value, 2, '.', ',');
      $row[] = number_format($threshold_value, 0);
    }  
    else {
      $row[] = 0;
      $row[] = 0;
    }
   
   //Admin User Data
   if ($display_user_data) {
     $row[] = $display_user_data;
   }
   else {
     $row[] = 'N/A';
   }
    
   if (user_access('configure group budget')) {
     $row[] = l('Edit', 'admin/settings/cloud_metering/group_settings/' . $g['rid'] . '/edit' );
   }
   else {
     $row[] = '--';
   }
   $rows[] = $row;
   
  }
  $table = theme('table', $headers, $rows);
  
  return $table;
}

/**
 *  Set the cost threshold for the given instance
 *  @param string $cloud_context The cloud context
 *  @param string $instance_id The instance Id for which to set the threshold
 *  @return array The form array
 */
function cloud_metering_set_threshold($form_state) {
  
  $cloud_context = _cloud_metering_filter_input(arg(4));
  $instance_id   = _cloud_metering_filter_input(arg(5));;
  
  // Check whether this instance exists
  $instance_exists = FALSE;
  $instances = cloud_get_instances($cloud_context);
  if (is_array($instances)) {
    foreach ($instances as $key => $instance) {
      if ($instance_id == $key) {
        $instance_exists = TRUE;
        break;
      }
    } 
  }
  
  if (!$instance_exists) {
    drupal_set_message(t('@instance_id is not a valid instance.', array('@instance_id' => $instance_id)), 'warning');
    drupal_goto('admin/settings/cloud_metering');
  }
  
  $budget_data = _cloud_metering_get_instance_budget($cloud_context, $instance_id);
    
  $get_instance_budget = $budget_data['budget'];
  $threshold           = $budget_data['threshold'];
  
  // Check instance lock status
  $module      = cloud_get_module($cloud_context);
  $lock_status = cloud_perform_action('', 'get_instance_lock_status', $cloud_context, array('instance_id' => $instance_id));
  $locked      = ($lock_status == 'Lock' ? TRUE : FALSE);
  
  $form = array();
  
  $form['wrapper'] = array(
    '#type' => 'fieldset',
    '#description' => $locked ? t('This instance is locked. It must be unlocked for setting budget.') : '',
    '#title' => '',
  );
  
  $form['wrapper']['initial_budget'] = array(
      '#type' => 'textfield',
      '#title' => t('Initial Budget ($) for @instance_id', array('@instance_id' => $instance_id)),
      '#description' => t('Enter Zero(0) for unlimited budget.'),      
      '#default_value' => $get_instance_budget,
      '#size' => 20,
      '#disabled' => $locked,
  );
  
  $form['wrapper']['cost_threshold'] = array(
    '#title' => t('Threshold (%) for @instance_id', array('@instance_id' => $instance_id)),
    '#description' => t('The cost threshold percentage. Thershold value will be percent of the initial budget.'),
    '#type' => 'textfield',
    '#default_value' => $threshold,
    '#size' => 20,
    '#disabled' => $locked,
  );
  
  $form['wrapper']['cloud_context'] = array(
    '#type' => 'value',
    '#value' => $cloud_context,
  );
  
  $form['wrapper']['instance_id'] = array(
    '#type' => 'value',
    '#value' => $instance_id,
  );
  
  $form['wrapper']['submit'] = array(
    '#type' => 'submit',
    '#submit' => array('_cloud_metering_update_instance_budget'),
    '#value' => t('Save'),
    '#disabled' => $locked,
  );
  
  return $form;
}

/**
 *  Validation handler for cloud_metering_set_threshold
 *  @param array $form Form object
 *  @param array $form_state Form values
 */
function cloud_metering_set_threshold_validate($form, &$form_state) {
  // Check whether it's a valid instance
  
  // Check the entered threshold value
  $threshold = $form_state['values']['cost_threshold'];
  if (!is_numeric($threshold)) {
    form_set_error('threshold', t('Please enter a numeric value for the threshold'));
  }
  if (is_numeric($threshold) && $threshold < 0) {
    form_set_error('threshold', t('The threshold value should not be less than 0'));
  }
  
  $initial_budget  = $form_state['values']['initial_budget'];  
  
  if (!is_numeric($initial_budget)) {    
    form_set_error('initial_budget', t('Please enter a numeric value in Initial Budget'));
  }
  if (is_numeric($initial_budget) && $initial_budget < 0) {
    form_set_error('initial_budget', t('The Initial Budget value should not be less than 0'));
  }
}

/**
 *  Replace mail text tokens
 */
function cloud_metering_mail_replacements($key, $language = NULL, $variables = array()) {
  
  $langcode = isset($language) ? $language->language : NULL;

  if ($admin_setting = variable_get(CLOUD_METERING_MODULE_NAME . '_' . $key, FALSE)) {
    return strtr($admin_setting, $variables);
  }
  else {
    switch ($key) {
      case 'threshold_mail_subject':
        return t(CLOUD_METERING_DEFAULT_THRESHOLD_MAIL_SUBJECT, $variables);
      case 'threshold_mail_template':
        return t(CLOUD_METERING_DEFAULT_THRESHOLD_MAIL_BODY, $variables);
      case 'terminate_mail_subject':
        return t(CLOUD_METERING_DEFAULT_TERMINATE_MAIL_SUBJECT, $variables);
      case 'terminate_mail_template':
        return t(CLOUD_METERING_DEFAULT_TERMINATE_MAIL_BODY, $variables);
      case 'user_mail_subject':
        return t(CLOUD_METERING_DEFAULT_USER_MAIL_SUBJECT, $variables);
      case 'user_mail_template':
        return t(CLOUD_METERING_DEFAULT_USER_MAIL_BODY, $variables);
      case 'group_mail_subject':
        return t(CLOUD_METERING_DEFAULT_GROUP_MAIL_SUBJECT, $variables);
      case 'group_mail_template':
        return t(CLOUD_METERING_DEFAULT_GROUP_MAIL_BODY, $variables);
    }
  }

}

/**
 *  Menu callback
 */
function cloud_metering_mail_settings($form_state) {
  
  $form = array();
  
  $description = t('Customize the format of the mail that will be sent whenever an instance crosses the cost threshold. The following tokens are available -');
  $description .= '<br />';

  $tokens = cloud_metering_mail_tokens();
  foreach ($tokens as $token => $token_detail) {
    $description .= $token . ' - ' . $token_detail['description'] . '<br />';
  }
  
  $form['wrapper'] = array(
    '#type' => 'fieldset',
    '#title' => t('Mail Settings'),
    '#description' => t('The metering module sends out notification mails whenever a cloud instance\'s cost has reached the budget threshold or when an instance\'s cost has gone past the budget allocated and needs to be terminated. The format of the mails can be configured using the following forms.'),
    '#collapsible' => FALSE,
  );
  
  $form['wrapper']['threshold_mails'] = array(
    '#type' => 'fieldset',
    '#title' => t('Instance cost reached or crossed threshold'),
    '#description' => $description,
    '#collapsible' => TRUE,
  );
  
  $default_subject = t(CLOUD_METERING_DEFAULT_THRESHOLD_MAIL_SUBJECT);
  $default_subject = variable_get('cloud_metering_mail_subject', $default_subject);
  
  $form['wrapper']['threshold_mails']['cloud_metering_mail_subject'] = array(
    '#type' => 'textfield',
    '#title' => t('Subject'),
    '#description' => t('The mail subject'),
    '#default_value' => $default_subject,
  );
  
  $default_template = t(CLOUD_METERING_DEFAULT_THRESHOLD_MAIL_BODY);
  $default_template = variable_get('cloud_metering_mail_template', $default_template);
  
  $form['wrapper']['threshold_mails']['cloud_metering_mail_template'] = array(
    '#type' => 'textarea',
    '#title' => t('Body'),
    '#description' => t('The mail body'),
    '#default_value' => $default_template,
  );
  
  $form['wrapper']['terminate_mails'] = array(
    '#type' => 'fieldset',
    '#title' => t('Instance cost reached or crossed budget'),
    '#description' => $description,
    '#collapsible' => TRUE,
    '#collapsed'   => TRUE,
  );
  
  $default_subject = t(CLOUD_METERING_DEFAULT_TERMINATE_MAIL_SUBJECT);
  $default_subject = variable_get('cloud_metering_terminate_mail_subject', $default_subject);
  
  $form['wrapper']['terminate_mails']['cloud_metering_terminate_mail_subject'] = array(
    '#type' => 'textfield',
    '#title' => t('Subject'),
    '#description' => t('The mail subject'),
    '#default_value' => $default_subject,
  );
  
  $default_template = t(CLOUD_METERING_DEFAULT_TERMINATE_MAIL_BODY);
  $default_template = variable_get('cloud_metering_terminate_mail_template', $default_template);
  
  $form['wrapper']['terminate_mails']['cloud_metering_terminate_mail_template'] = array(
    '#type' => 'textarea',
    '#title' => t('Body'),
    '#description' => t('The mail body'),
    '#default_value' => $default_template,
  );
  
  $form['wrapper']['user_mails'] = array(
    '#type' => 'fieldset',
    '#title' => t('Total cost of all instances of user reached or crossed budget'),
    '#description' => $description,
    '#collapsible' => TRUE,
    '#collapsed'   => TRUE,
  );
  
  $default_subject = t(CLOUD_METERING_DEFAULT_USER_MAIL_SUBJECT);
  $default_subject = variable_get('cloud_metering_user_mail_subject', $default_subject);
  
  $form['wrapper']['user_mails']['cloud_metering_user_mail_subject'] = array(
    '#type' => 'textfield',
    '#title' => t('Subject'),
    '#description' => t('The mail subject'),
    '#default_value' => $default_subject,
  );
  
  $default_template = t(CLOUD_METERING_DEFAULT_USER_MAIL_BODY);
  $default_template = variable_get('cloud_metering_user_mail_template', $default_template);
  
  $form['wrapper']['user_mails']['cloud_metering_user_mail_template'] = array(
    '#type' => 'textarea',
    '#title' => t('Body'),
    '#description' => t('The mail body'),
    '#default_value' => $default_template,
  );
  
  $form['wrapper']['group_mails'] = array(
    '#type' => 'fieldset',
    '#title' => t('Total cost of all instances of group reached or crossed budget'),
    '#description' => $description,
    '#collapsible' => TRUE,
    '#collapsed'   => TRUE,
  );
  
  $default_subject = t(CLOUD_METERING_DEFAULT_GROUP_MAIL_SUBJECT);
  $default_subject = variable_get('cloud_metering_group_mail_subject', $default_subject);
  
  $form['wrapper']['group_mails']['cloud_metering_group_mail_subject'] = array(
    '#type' => 'textfield',
    '#title' => t('Subject'),
    '#description' => t('The mail subject'),
    '#default_value' => $default_subject,
  );
  
  $default_template = t(CLOUD_METERING_DEFAULT_GROUP_MAIL_BODY);
  $default_template = variable_get('cloud_metering_group_mail_template', $default_template);
  
  $form['wrapper']['group_mails']['cloud_metering_group_mail_template'] = array(
    '#type' => 'textarea',
    '#title' => t('Body'),
    '#description' => t('The mail body'),
    '#default_value' => $default_template,
  );
  
  return system_settings_form($form);
  
}

/**
 * Returns a number of tokens that can be used in the mail template
 * Optionally returns the replacement values for the tokens
 * @param boolean $values Whether to return the token values
 * @param array $tokens The tokens
 * @param string $cloud_context The cloud context
 * @param string $instance_id The instance id
 * @param string $instance_name The instance name
 * @param object $user An user object
 * @return array The array of tokens
 */
function cloud_metering_mail_tokens($values = FALSE, $cloud_context = '', $instance_id = '', $instance_name = '', $user = NULL) {
  
  global $base_url;
  
  $tokens = array();
  
  if ($values) {
    if (empty($cloud_context) || empty($instance_id)) {
      drupal_set_message(t('Token values can\'t be returned without a cloud context and an instance id.'), 'warning');
      $values = FALSE;
    }
    elseif (empty($user)) {
      $user = user_load(array('uid' => 1));
    }
  }
  
  $tokens = array();
  
  $value                      = $values ? $user->name : '';
  $tokens['!username']        = array('description' => t('Username'), 'value' => $value);
  
  $tokens['!site']            = array('description' => t('The site-name'), 'value' => variable_get('site_name', 'Drupal'));
  
  $value                      = $values ? $instance_id : '';
  $tokens['!instance-id']     = array('description' => t('The instance Id'), 'value' => $value);
  
  $value                      = $values ? $instance_name : '';
  $tokens['!instance-name']   = array('description' => t('The Instance name'), 'value' => $value);
  
  $value                      = $values ? url('clouds/' . $cloud_context . '/instances/terminate', array('query' => array('instance_id' => $instance_id))) : '';
  $tokens['!terminate-url']   = array('description' => t('The URL for terminating an instance'), 'value' => $value);
  
  return $tokens;
  
}

/**
 *  Get instances whose costs have reached the threshold values
 */
function cloud_metering_threshold_crossed_instances($cloud_context = NULL) {
  
  $peak_instances = array();
  
  if (empty($cloud_context)) {
    $clouds = cloud_get_all_clouds();
  }
  else {
    $clouds = array($cloud_context);
  }

  $end_date   = date('c');
  
  foreach ($clouds as $cloud) {
    $instances = cloud_get_instances($cloud);
    
    foreach ($instances as $instance_id => $instance) {
      $runtime        = $instance['runtime'];
      $start_date     = date('c', strtotime($instance['runtime']));
      $initial_budget = _cloud_metering_get_instance_budget($cloud, $instance_id);
      if (is_numeric($initial_budget) && $initial_budget > 0) {
        // The following introduces a dependency on cloud_dashboard
        // The generate data function should be in metering module
        $instance_data = _cloud_metering_instance_cost_data($start_date, $end_date, $cloud, $instance_id);
        $cost          = 0;
        if (is_array($instance_data)) {
          $linux_or_unix_usage = $instance_data['linux_or_unix_usage'];
          $total_hours         = _cloud_metering_instance_runtime($runtime);
          $cost                = $linux_or_unix_usage * $total_hours;
        }
        $threshold = _cloud_metering_get_threshold($cloud, $instance_id);
        $threshold = ( $initial_budget * $threshold ) / 100;
        // If cost has gone past the budget then this instance needs to be terminated and considered separately
        if ( $cost > 0 && $threshold > 0 && $cost < $initial_budget && $cost > $threshold ) {
          $peak_instances[] = $instance;
        } 
      }
    }
  }
  
  return $peak_instances;
}

/**
 *  Get the instances whose costs have reached the set budget value
 */
function cloud_metering_maxed_instances($cloud_context = NULL) {
  
  $maxed_instances = array();
  
  if (empty($cloud_context)) {
    $clouds = cloud_get_all_clouds();
  }
  else {
    $clouds = array($cloud_context);
  }

  $end_date   = date('c');
  
  foreach ($clouds as $cloud) {
    $instances = cloud_get_instances($cloud);
    
    foreach ($instances as $instance_id => $instance) {
      $runtime        = $instance['runtime'];
      $start_date     = date('c', strtotime($instance['runtime']));
      $budget_data    = _cloud_metering_get_instance_budget($cloud, $instance_id);
      $initial_budget = $budget_data['budget'];
      if (is_numeric($initial_budget) && $initial_budget > 0) {
        $instance_data = _cloud_metering_instance_cost_data($start_date, $end_date, $cloud, $instance_id);
        $cost          = 0;
        if (is_array($instance_data)) {
          $linux_or_unix_usage = $instance_data['linux_or_unix_usage'];
          $total_hours         = _cloud_metering_instance_runtime($runtime);
          $cost                = $linux_or_unix_usage * $total_hours;
        }
        if ($cost > $initial_budget) {
          $maxed_instances[] = $instance;
        } 
      }
    }
  }
  
  return $maxed_instances;
  
}

/**
 *  Send notifications
 */
function cloud_metering_notify_instance_users($instances) {
  
  foreach ($instances as $instance) {
    $cloud_context = $instance['cloud_context'];
    $instance_id   = $instance['instance_id'];
    $instance_name = $instance['name'];
    
    // Check if a notification has already been sent for this instance
    $notification_sent = cloud_metering_notification_sent('threshold', $cloud_context, $instance_id);
    
    if (!$notification_sent) {
      // Send a notification
      $user = _cloud_metering_get_instance_user($instance);
      
      $mail_result = drupal_mail(
        CLOUD_METERING_MODULE_NAME,
        'threshold_mail',
        $user->mail,
        user_preferred_language($user),
        array('user' => $user, 'cloud_context' => $cloud_context, 'instance_id' => $instance_id, 'instance_name' => $instance_name),
        variable_get('site_mail', ini_get('sendmail_from')),
        TRUE
      );
      
      if ($mail_result['result']) {
        cloud_metering_save_notification_info('threshold', $cloud_context, $instance_id);
      }
      else {
        _cloud_metering_activity_audit(t('Failed to send notification mail for instance %instance_id', array('%instance_id' => $instance_id)), 'user_activity', '', WATCHDOG_ERROR);
      }
    }
  }
}

/**
 *  Terminate instances whose costs have gone past allocated budget
 *  @param String maxed_instances 
 */
function cloud_metering_terminate_budget_crossed_instances($maxed_instances) {
  
  foreach ($maxed_instances as $instance) {
    
    if ($instance['state'] == 'running') {
      
      // Backup->Detach->Terminate the instance
      
      $cloud_context = $instance['cloud_context'];
      $instance_id   = $instance['instance_id'];
      
      $instance_volumes = cloud_get_instance_volumes($cloud_context, $instance_id);
      $volumes          = array();
      
      if (is_array($instance_volumes) && sizeof($instance_volumes) > 0) {
        
        foreach ($instance_volumes as $volume_id => $volume) {
          $volumes[$volume_id] = array();
        }
        
        $data = array('volumes' => $volumes);
        // Initiate a backup task
        _cloud_metering_add_task($cloud_context, $instance_id, CLOUD_METERING_BACKUP, $data);
        
      }
      else {
        
        // Initiate terminate
        _cloud_metering_add_task($cloud_context, $instance_id, CLOUD_METERING_TERMINATE, array());
        
      }
      
      
      // Send a notification
      $notification_sent = cloud_metering_notification_sent('terminate', $cloud_context, $instance_id);
      
      if (!$notification_sent) {
        
        $instance_name = $instance['name'];
      
        // The Drupal associated with the instance
        $user = _cloud_metering_get_instance_user($instance);
        
        $mail_result = drupal_mail(
          CLOUD_METERING_MODULE_NAME,
          'terminate_mail',
          $user->mail,
          user_preferred_language($user),
          array('user' => $user, 'cloud_context' => $cloud_context, 'instance_id' => $instance_id, 'instance_name' => $instance_name),
          variable_get('site_mail', ini_get('sendmail_from')),
          TRUE
          
        );
        
        if ($mail_result['result']) {
          cloud_metering_save_notification_info('terminate', $cloud_context, $instance_id);
        }
        else {
          _cloud_metering_activity_audit(t('Failed to send notification mail for instance %instance_id.', array('%instance_id' => $instance_id)), 'user_activity', '', WATCHDOG_ERROR);
        } 
        
      }
      
    }
    
  }
  
}

/**
 *  Get an instance
 */
function cloud_metering_get_instance($cloud_context, $instance_id) {
  
  $instances = cloud_get_instances($cloud_context);
  
  if (is_array($instances)) {
    
    foreach ($instances as $instance) {
      if ($instance['instance_id'] == $instance_id) {
        return $instance;
      }
    }
    
  }
  
  return FALSE;
  
}

/**
 *  Run through all the instances and return as different groups
 */
function cloud_metering_get_grouped_instances($cloud_context = NULL) {
  
  $return                                = array();
  $return['threshold_crossed_instances'] = array();
  $return['budget_crossed_instances']    = array();
  $return['user_instances']              = array();
  $return['group_instances']             = array();
  
  $groups = _cloud_metering_get_groups();
  
  foreach ($groups as $group) {
    $return['group_instances'][$group['grid']] = array('gid' => $group['gid'], 'instances' => array());
  }
  
  if (empty($cloud_context)) {
    $clouds = cloud_get_all_clouds();
  }
  else {
    $clouds = array($cloud_context);
  }

  $end_date   = date('c');
  
  foreach ($clouds as $cloud) {
    
    $instances = cloud_get_instances($cloud);
    
    foreach ($instances as $instance_id => $instance) {
      
      $runtime        = $instance['runtime'];
      $start_date     = date('c', strtotime($instance['runtime']));
      $budget_data    = _cloud_metering_get_instance_budget($cloud, $instance_id);
      $initial_budget = $budget_data['budget'];
      $threshold      = $budget_data['threshold'];
      
      $instance['budget']    = $initial_budget;
      $instance['threshold'] = $threshold;
      
      if (is_numeric($initial_budget) && $initial_budget > 0) {
        
        $instance_data = _cloud_metering_instance_cost_data($start_date, $end_date, $cloud, $instance_id);
        $cost          = 0;
        
        if (is_array($instance_data)) {
          $linux_or_unix_usage = $instance_data['linux_or_unix_usage'];
          $total_hours         = _cloud_metering_instance_runtime($runtime);
          $cost                = $linux_or_unix_usage * $total_hours;
        }
        
        $instance['cost'] = $cost;
        
        $threshold = ( $initial_budget * $threshold ) / 100;
        
        if ($cost > 0 && $threshold > 0 && $cost < $initial_budget && $cost > $threshold) {
          $return['threshold_crossed_instances'][] = $instance;
        } 
        elseif ($cost > $initial_budget) {
          $return['budget_crossed_instances'][]    = $instance;
        }
        
        $instance_user = _cloud_metering_get_instance_user($instance);
        $uid           = $instance_user->uid;
        if (!array_key_exists($uid, $return['user_instances'])) {
          $return['user_instances'][$uid] = array();
        }
        $return['user_instances'][$uid][] = $instance;
        
        if ($roles = $instance_user->roles) {
          foreach ($roles as $rid => $role) {
            if (array_key_exists($rid, $return['group_instances'])) {
              $return['group_instances'][$rid]['instances'][] = $instance;
            }
          }
        }
        
      }
      
    }
    
  }
  
  return $return;
  
}

function cloud_metering_notify_group_admins($group_instances) {
  
  foreach ($group_instances as $rid => $data) {
    
    $gid       = $data['gid'];
    $instances = $data['instances'];
    
    // Get group budget and compare with the total cost
    $group_budget = _cloud_metering_get_group_budget(array('column' => 'gid', 'value' => $gid));
    
    if (empty($group_budget)) {
      continue;
    }
    
    $group_budget = $group_budget[$gid];
    $group_threshold = $group_budget['threshold'];
    $group_budget = $group_budget['group_budget'];
    
    if ($group_budget > 0 && $group_threshold > 0) {
      
      $cost = 0;
      foreach ($instances as $instance) {
        $cost += $instance['cost'];
      }
      
      if ($cost >= ($group_budget * $group_threshold / 100)) {
        // Send notification to group admins
        $group_admins = _cloud_metering_get_group_admins($gid);
        foreach ($group_admins as $group_admin) {
          $account = user_load(array('uid' => $group_admin));
          $mail    = $account->mail;
          $mail_result = drupal_mail(
            CLOUD_METERING_MODULE_NAME,
            'group_mail',
            $mail,
            user_preferred_language($account),
            array('user' => $account, 'cloud_context' => '', 'instance_id' => '', 'instance_name' => ''),
            variable_get('site_mail', ini_get('sendmail_from')),
            TRUE
          );
        }
      }
      
    }
    
  }
  
}

function cloud_metering_notify_users($user_instances) {
  
  foreach ($user_instances as $uid => $instances) {
    
    // Get user budget and compare with the total cost
    $user_budget = _cloud_metering_get_user_budget($uid);
    
    if ($user_budget > 0) {
      
      $cost = 0;
      foreach ($instances as $instance) {
        $cost += $instance['cost'];
      }
      
      if ($cost >= $user_budget)  {
        // Send notification to user
        $account = user_load(array('uid' => $uid));
        $mail    = $account->mail;
        $mail_result = drupal_mail(
            CLOUD_METERING_MODULE_NAME,
            'group_mail',
            $mail,
            user_preferred_language($account),
            array('user' => $account, 'cloud_context' => '', 'instance_id' => '', 'instance_name' => ''),
            variable_get('site_mail', ini_get('sendmail_from')),
            TRUE
          );
        
      }
      
    }
    
  }
  
}

function cloud_metering_add_checker_tasks() {
  
  $in_progress_tasks = _cloud_metering_get_in_progress_tasks();
  
  if (is_array($in_progress_tasks)) {
    
    foreach ($in_progress_tasks as $in_progress_task) {
      
      $data = unserialize($in_progress_task['data']);
      $cloud_context = $data['cloud_context'];
      $instance_id = $data['instance_id'];
      $task = $in_progress_task['task'];
      
      if ($task == CLOUD_METERING_BACKUP) {
        $task = CLOUD_METERING_BACKUP_CHECK;
      }
      elseif ($task == CLOUD_METERING_DETACH) {
        $task = CLOUD_METERING_DETACH_CHECK;
      }
      elseif ($task == CLOUD_METERING_TERMINATE) {
        $task = CLOUD_METERING_TERMINATE_CHECK;
      }
      
      if (_cloud_metering_get_task_type($task) == 'base') {
        continue;
      }
      $data['qid'] = $in_progress_task['qid'];
      // Add a checker task
      _cloud_metering_add_task($cloud_context, $instance_id, $task, $data);
      
    }
    
  }
  
}

/**
 * Get Druapl Role details as Group details
 */
function _cloud_metering_get_group_details($arg = array()) {

  $rows = array();
  $params = array();
  
  if (!empty($arg['value'])) {
    $where_query = 'WHERE ' . $arg['column'] . ' = %d';
    $params[] = $arg['value'];
  }
  else {
    $where_query = 'WHERE rid > 2';    
  }
    
  $group_query = db_query('SELECT * FROM {role} ' . $where_query, $params);
  
  while ($g = db_fetch_object($group_query)) {
   $row = array();
      
   $row['rid'] = $g->rid;
   $row['name'] = $g->name;
   
   if (!empty($arg)) {
     $rows[$arg['value']] = $row;
   }
   else {
     $rows[$g->rid] = $row;
   }
  }
  
  return $rows;
}

/**
 * Get Group Budget details
 */
function _cloud_metering_get_group_budget($arg) {
  
  $rows = array();
  $params = array();
  
  if (!empty($arg['value'])) {
    $where_query = 'WHERE ' . $arg['column'] . ' = %d';
    $params[] = $arg['value'];
  }
  else {
    $where_query = '';    
  }
  
  $group_query = db_query('SELECT * FROM {' . CLOUD_METERING_BUDGET_GROUP_TABLE . '} ' . $where_query, $params);
  
  while ($g = db_fetch_object($group_query)) {
    $row = array();
   
    $row['grid'] = $g->grid;   
   
    $row['threshold'] = $g->threshold;
    $row['group_budget'] = $g->group_budget;
    $row['user_data'] = $g->user_data;
   
    if (!empty($arg)) {
      $rows[$arg['value']] = $row;
    }
    else {
      $rows[$g->rid] = $row;
    }
  }
  
  return $rows;
}

/**
 * Get User Allocated Budget 
 */
 
function _cloud_metering_get_user_budget($arg) {
  
  $budget_query = db_result(db_query('SELECT budget FROM {' . CLOUD_METERING_USER_BUDGET_TABLE . '} WHERE uid = %d', $arg));
  
  if (is_numeric($budget_query)) {
    return $budget_query;
  }
  else {
    return 0;
  }
}


/**
 * Get Group Users Details 
 */
 
function _cloud_metering_get_group_users_details($arg='') {
  
  if (!empty($arg['value'])) {
    $where_query = 'WHERE ' . $arg['column'] . ' = %d';
  }
  else {
    $where_query = '';
  }
  
  $group_users_query = db_query('SELECT * FROM {' . CLOUD_METERING_USER_BUDGET_TABLE . '} ' . $where_query, $arg['value']);
  
  $group_users = array();
  
  while ($u = db_fetch_object($group_users_query)) {
     $row = array();
     $row['uid'] = $u->uid;
     $row['gid'] = $u->gid;
     $row['budget'] = $u->budget;
     
     if (!empty($arg)) {
       $group_users[$u->uid] = $row;
     }
     else {
       $group_users[$u->gid] = $row;
     }
  }
  
  return $group_users;
}

/**
 * Get all Drupal Users where user id greater than 1
 */
 
function _cloud_metering_get_users_list_by_role($rid) {
  
  $users = array();
   
  $users_query = db_query('SELECT a.uid, a.mail, a.name, b.rid FROM {users} a, {users_roles} b WHERE a.uid = b.uid and b.rid = %d', $rid);
  
  while ($u = db_fetch_object($users_query)) {
   $user = array();
   
   $user['id'] = $u->uid;
   $user['name'] = $u->name;
   $user['mail'] = $u->mail;
   $user['rid'] = $u->rid;
      
   $users[$u->uid] = $user;
   
  }
  
  return $users;
}

/**
 * Get all Drupal Roles expect the Default Drupal roles
 */
 
function _cloud_metering_get_roles_list($arg='') {
  
  $roles = array();
  $roles[0] = '--Select Role--';
  
  if (!empty($arg)) {
    $where_query = 'WHERE rid =' . $arg;
  }
  else {
    $where_query = 'WHERE rid > 2';
  }
  
  $roles_query = db_query('SELECT * FROM {role} ' . $where_query);
  
  while ($u = db_fetch_object($roles_query)) {
   $roles[$u->rid] = $u->name;
  }
  
  return $roles;
}

/**
 * Add/Edit User budget for Groups
 */
function _cloud_metering_add_users_to_group($form_state) {
  global $user;
  
  $user_id = filter_xss(arg(5));
  $operation = filter_xss(arg(6));
  $user_data = user_load($user_id);
    
  if (!user_access('configure user budget')) {   
    drupal_set_message(t('You do not have permission to Edit the Group Budget or the User ID is invalid!'));
    drupal_goto('admin/settings/cloud_metering');
  }
  elseif (!empty($user_id) && empty($user_data)) {
    drupal_set_message(t('You do not have permission to Edit the Group Budget or the User ID is invalid!'));
    drupal_goto('admin/settings/cloud_metering/group_users');
  }
  else {
    $arg = array('column' => 'uid', 'value' => $user_id);
    $group_user_details = _cloud_metering_get_group_users_details($arg);
  }
    
  $group_details = _cloud_metering_get_group_details();
  $group_list[0] = t('--Select Group--');
  
  foreach ($group_details as $key => $values) {
    $group_list[$key] = $values['name'];
  }
  
  $form = array();
  
  $form['wrapper'] = array(
    '#type' => 'fieldset',
    '#title' => '',
  );
  
  if (!user_access('configure group budget')) {
    $disabled = array('disabled' => TRUE);
  }
  else {
    $disabled = '';
  }
  
  $form['wrapper']['user_group'] = array(
    '#type' => 'select',
    '#title' => t('Select a Group'),
    '#description' => t('Please select a Group to Associate with the Selected User'),
    '#options' => $group_list,
    '#default_value' => $group_user_details[$user_id]['gid'],
    '#attributes' => $disabled,
    '#weight' => 3
  );
  
  $form['wrapper']['user_budget'] = array(
    '#type' => 'textfield',
    '#title' => t('User Budget ($)'),
    '#default_value' => $group_user_details[$user_id]['budget'],
    '#description' => t('Enter the User Budget Value. Put only the numeric value without a currency symbol.'),
    '#required' => TRUE,
    '#weight' => 4
  );
  
  if ($operation == 'edit') {
    $submit_value = 'Update';
        
    $form['user_id'] = array(
      '#type' => 'hidden',
      '#value' => $user_id,
    );
    
    $form['current_budget'] = array(
      '#type' => 'hidden',
      '#value' => _cloud_metering_get_user_budget($user_id),
    );
    
    $form['wrapper']['remove_user'] = array(
      '#type' => 'checkbox',
      '#title' => t('Remove the User from the Group'),
      '#weight' => 5
    );
    
  }
  elseif (user_access('configure group budget')) {
    $form['wrapper']['user_id'] = array(
      '#type' => 'select',
      '#title' => t('Select a User'),
      '#description' => t('Please select a User to be Added to a Group'),
      '#options' => _cloud_metering_get_users_list_by_role(),
      '#weight' => 1
    );
  
    $submit_value = t('Add');
  }
  else {
    drupal_set_message('You do not have permission to Add User to Group');
    drupal_goto(drupal_get_path_alias('user/'. $user->uid));
  }
  
  $form['wrapper']['submit'] = array(
    '#type' => 'submit',
    '#value' => $submit_value,
    '#weight' => 6
  );
   
  return $form;
}

/**
 * Validating form for adding users to groups
 */
function _cloud_metering_add_users_to_group_validate($form, &$form_state) {
  $values = $form_state['values'];  
  $budget = $values['user_budget'];
  $user_id = $values['user_id'];  
  $group_id = $values['user_group'];
  $remove_user = $values['remove_user'];
    
  if ($remove_user) {
    return;
  }
  
  if ($user_id == 0) {
    form_set_error('user_id', t('Please select an User from the select list.'));
  }
  if ($group_id == 0) {
    form_set_error('user_group', t('Please select a Group Name from the select list.'));
  }
  
  $group_budget = db_result(db_query('SELECT group_budget FROM {' . CLOUD_METERING_BUDGET_GROUP_TABLE . '} WHERE gid = %d', $group_id));  
  
  //Validate user budget value  
  $get_budget_check = _cloud_metering_check_budget_changes($group_id, NULL, $user_id, $budget);
  
  if (!is_numeric($budget)) {
    form_set_error('user_budget', t('Please enter only numeric values only.'));
  }  
  elseif ($budget <= 0) {
    form_set_error('user_budget' , t('Please enter values greater than Zero(0)'));
  }  
  elseif ($get_budget_check == 1) {
    form_set_error('user_budget' , t('User Budget Value is greater than Group Budget. Please enter a User Budget lesser than Group Budget.'));
  }
  elseif ($get_budget_check == 2) {
    form_set_error('user_budget' , t('User Budget Value is greater than Group Users Total Budget. Please enter a User Budget lesser than Group Users Total Budget.'));
  }
}

/**
 * Submiting the values of the Update User Budget form
 */
function _cloud_metering_add_users_to_group_submit($form, &$form_state) {
  
  $values = $form_state['values'];
  $user_id = $values['user_id'];  
  $budget = $values['user_budget'];
  $group_id = $values['user_group'];
  $remove_user = $values['remove_user'];
  
  //Get User Budget value
  $user_budget = _cloud_metering_get_user_budget($user_id);
  
  if ($user_budget == 0) {
    $update_budget = db_query('INSERT INTO {' . CLOUD_METERING_USER_BUDGET_TABLE . '} (id, uid, gid, budget) VALUES (NULL, %d, %d, %f)', $user_id, $group_id, $budget);
    
    if ($update_budget) {
      drupal_set_message(t('User is removed from the group.'));
      drupal_goto('admin/settings/cloud_metering/group_users');
    }
  } 
  elseif ($user_budget > 0 ) {
    
    if ($remove_user) {
      $update_budget = db_query('DELETE FROM {' . CLOUD_METERING_USER_BUDGET_TABLE . '} WHERE uid = %d', $user_id);
      
      if ($update_budget) {
        drupal_set_message(t('User is removed from the group.'));
        drupal_goto('admin/settings/cloud_metering/group_users');
      }
    }
    else {
      $update_budget = db_query('UPDATE {' . CLOUD_METERING_USER_BUDGET_TABLE . '} SET budget = %f , gid = %d WHERE uid=%d', $budget, $group_id, $user_id);
      
      if ($update_budget) {
        drupal_set_message(t('User budget is updated.'));
        drupal_goto('admin/settings/cloud_metering/group_users/' . $group_id);
      }
    }  
  }
  
}

/**
 * Function validating initial budget value for individual Templates
 */
function _cloud_metering_validate_template_budget($form, &$form_state) {
  global $user;
  $error = 0;
  $form_values = $form_state['values'];
  
  $initial_budget  = $form_values['initial_budget'];
  $threshold       = $form_values['cost_threshold'];
     
  if (!is_numeric($initial_budget)) {
    $error = 1;
    form_set_error('initial_budget', t('Please enter a numeric value in Initial Budget'));
  }
  if (is_numeric($initial_budget) && $initial_budget < 0) {
    form_set_error('initial_budget', t('The Initial Budget value should not be less than 0'));
  }
  
  if (!is_numeric($threshold)) {
    $error = 1;
    form_set_error('cost_threshold', t('The cost threshold should be numeric.'));
  }
  if (is_numeric($threshold) && $threshold < 0) {
    form_set_error('cost_threshold', t('The threshold should not be less than 0.'));
  }
  
}

/**
 *  Submit handler for cloud_metering_set_threshold and cloud_metering_form_alter
 *  @param array $form Form object
 *  @param array $form_state Form values
 *  
 */
function _cloud_metering_update_instance_budget($form, &$form_state) {
  global $user;
  $error = 0;
  $form_values = $form_state['values'];
  
  $instance_id     = $form_values['instance_id'];
  $cloud_context   = $form_values['cloud_context'];
  $initial_budget  = $form_values['initial_budget'];
  $threshold       = round($form_values['cost_threshold'], 2);
  
  // Proceed to save only if instance is not locked
  $lock_status     = cloud_perform_action('', 'get_instance_lock_status', $cloud_context, array('instance_id' => $instance_id));
  $locked          = ($lock_status == 'Lock' ? TRUE : FALSE);
  
  if ($locked) {
    form_set_error('initial_budget', t('Budget settings could not be saved as the instance is locked.'));
    return;
  }
  
  $instance_details = cloud_metering_get_instance($cloud_context, $instance_id);
  
  $account = _cloud_metering_get_instance_user($instance_details); 
  
  if (!is_numeric($initial_budget)) {
    $error = 1;
    form_set_error('initial_budget', t('Please enter a numeric value in Initial Budget'));
  }
  else {
    // Check the user's budget
    $user_budget = _cloud_metering_get_user_budget($account->uid);
    
    if ($user_budget > 0) {
      $allocated_budget = _cloud_metering_get_user_instances_budget($account->name);
      $total_budget = $allocated_budget + $initial_budget;
      if ($total_budget > $user_budget) {
        $error = 1;
        $total_budget = number_format($total_budget, 2, '.', ',');
        $user_budget  = number_format($user_budget, 2, '.', ',');
        $edit_link    = '';
        $rid          = '';
        if (!empty($account->roles) && user_access('configure user budget')) {
          foreach ($account->roles as $rid => $role) {
            if ($role != 'authenticated user') {
              break;
            }
          }
          
          if (!empty($rid)) {
            $edit_link = l(t('Edit user budget'), 'admin/settings/cloud_metering/group_settings/' . $rid . '/edit');
          }
        }
        $message = filter_xss( t("Instance budget settings not saved due to user's budget constraint. Setting this budget will make the budget allocated to all instances of the user @total; but the user's budget is @user-budget. !edit-link", array('@total' => $total_budget, '@user-budget' => $user_budget, '!edit-link' => $edit_link)) );
        drupal_set_message($message, 'warning');
      }
    }
  }
  
  if ($error == 0) {
    
    // Update records
    _cloud_metering_instance_budget_save($cloud_context, $instance_id, array('budget' => $initial_budget, 'threshold' => $threshold));
    
  }
}

/**
 *  Log watchdog messages as well as save them to cloud module's audit logs
 *  @param String message The message to save
 *  @param String type The type of message (Cloud Activity module only shows the user_activity type messages)
 *  @param String link A link, required by Cloud Activity Audit, not used by metering
 *  @param String serverity The severity of the message like notice, warning or error. Use watchdog severity constants
 */
function _cloud_metering_activity_audit($message, $type = 'user_activity', $link = '', $severity = WATCHDOG_NOTICE) {
  
  watchdog(CLOUD_METERING_MODULE_DISPLAY_NAME, $message, NULL, $severity);
  
  if (module_exists('cloud_activity_audit')) {
    
    cloud_activity_audit_log(array(
        'type'    => $type, 
        'message' => $message, 
        'link'    => $link,
        // 'severity' => $severity,
      )
    );
    
  }
  
  return TRUE;
  
}

/**
 *  Get the runtime in hours of a cloud instance
 *  @param $launch_time string The launch datetime
 *  @param $to_time string Optional "to" datetime
 *  @return integer The number of hours the instance has run
 */
function _cloud_metering_instance_runtime($launch_time, $to_time = '') {

  if (empty($launch_time)) {
    return 0 ;
  }

  $launch_time     = date_parse($launch_time) ;
  $gm_launch_time  = gmmktime( $launch_time['hour'], $launch_time['minute'], $launch_time['second'], $launch_time['month'], $launch_time['day'], $launch_time['year'] ) ;
  
  if (!empty($to_time)) {
    $to_time     = date_parse($to_time) ;
    $gm_to_time  = gmmktime( $to_time['hour'], $to_time['minute'], $to_time['second'], $to_time['month'], $to_time['day'], $to_time['year'] ) ;
  }
  else {
    $gm_to_time  = gmmktime() ;
  }
  
  $diff =  $gm_to_time - $gm_launch_time ;
  $hours = round($diff / (3600)) ;
  
  return $hours ;

}

/**
 *  Save template budget
 */
function _cloud_metering_update_template_budget($form, &$form_state) {
  
  $form_values = $form_state['values'];
  
  $template_id     = $form_values['templateid'];
  $initial_budget  = $form_values['initial_budget'];
  $threshold       = round($form_values['cost_threshold'], 2);
  
  _cloud_metering_template_budget_save( $template_id, array('budget' => $initial_budget, 'threshold' => $threshold) );
  
}

/**
 *  Get the instance user
 *  TODO - This only works for the AWS clouds. Need to do in cloud-agnostic way. [Abhijit - 8-16-11]
 */
function _cloud_metering_get_instance_user($instance) {
  
  $cloud_context = $instance['cloud_context'];
  $ssh_key_name  = $instance['user'];

  if (!empty($ssh_key_name)) {
    
    $key_owner = _cloud_metering_get_key_owner($cloud_context, $ssh_key_name);
      
    if (!empty($key_owner)) {
      
      $account = user_load(array('name' => $key_owner));
      
      if ($account && $account->uid) {
        return $account;          
      }
      
    }
  
  }
  
  $account = user_load(array('uid' => 1));
  
  return $account;
  
}

/**
 *  Get all instances associated with a user
 */
function _cloud_metering_get_user_instances($user_name, $clouds = array()) {
  
  $user_instances = array();
  
  if (empty($clouds)) {
    
    $clouds = cloud_get_all_clouds();
    
  }
  
  foreach ($clouds as $cloud) {
    
    $instances = cloud_get_instances($cloud);
    
    foreach ($instances as $instance) {
      $instance_user = _cloud_metering_get_instance_user($instance);
      if ($instance_user->name == $user_name) {
        $user_instances[] = $instance;
      }
    }
    
  }
  
  return $user_instances;
}

/**
 *  Get the total budget allocated to all instances of a user
 */
function _cloud_metering_get_user_instances_budget($user_name) {
  
  $user_instances = _cloud_metering_get_user_instances($user_name);
  
  $total_budget = 0;
  
  foreach ($user_instances as $user_instance) {
    
    $instace_id    = $user_instance['instance_id'];
    $cloud_context = $user_instance['cloud_context'];
    
    $budget        = _cloud_metering_get_instance_budget($cloud_context, $instace_id);
    
    if (!is_null($budget['instance_id'])) {
      $total_budget += $budget['budget'];
    }
    
  }

  return $total_budget;

}

/**
 * Check user budget while launching instances from server templates
 * @param String template_id
 *                            The Template-id
 * @param String cloud_context
 *                            The cloud context
 * @return Mixed
 *               Handover to launching routine in the original cloud module if budget is not a constraint
 *               Otherwise show an error message and return to templates listing page
 * TODO - Too many if-else statements - Need refactoring [Abhijit - 8-16-11]
 */
function _cloud_metering_check_launch($template_id, $cloud_context) {
  
  $launch = TRUE;
  
  $template_budget = _cloud_metering_get_template_budget($template_id);
  
  // Check if budget set for template
  if (!is_null($template_budget['template_id']) && $template_budget['budget'] !== 0) {
    
    $template_budget = $template_budget['budget'];
    
    if ($template_data = _cloud_metering_get_template_data($template_id)) {
      
      $key_name  = $template_data['key_name'];
      $instance_count = $template_data['count_instances'];
      $key_owner = _cloud_metering_get_key_owner($cloud_context, $key_name);    
      
      if ($key_owner) {
        $account = user_load(array('name' => $key_owner));
        
        if ($account->uid) {
          $user_budget = _cloud_metering_get_user_budget($account->uid);
          
          if ($user_budget > 0) {
            
            $total_allocated_budget = _cloud_metering_get_user_instances_budget($key_owner);
            $new_instances_total_budget = $template_budget * $instance_count;
            
            $total = $total_allocated_budget + $new_instances_total_budget;
            
            if ($total > $user_budget) {
              $total       = number_format($total, 2, '.', ',');
              $user_budget = number_format($user_budget, 2, '.', ',');
              $launch      = FALSE;
              $edit_link   = '';
              if (user_access('edit server template')) {
                $edit_link   = _cloud_metering_get_budget_config_link(array('cloud_context' => $cloud_context, 'template_id' => $template_id)); 
              }
              $message = filter_xss( t("Launching of instance(s) was stopped due to budget constraint. After launching instances from this template, the total budget allocated to user's instances will be @total; but the user's budget is @user-budget. !edit-link", array('@total' => $total, '@user-budget' => $user_budget, '!edit-link' => $edit_link)) );
              drupal_set_message($message, 'warning');
            }
            
          }
          
        }
        
      }
      
    }
    else {
      
      $launch = FALSE;
      drupal_set_message(t('Failed to launch instance. The server template could not be found.', 'error'));
      
    }
    
  }
  
  if ($launch) {
    // Handover to launching routine
    return cloud_perform_action($template_id, 'launch', $cloud_context);
  }
  else {
    drupal_goto( CLOUD_SERVER_TEMPLATES_PATH . '/list/' . $cloud_context );
  }
  
}

/**
 *  Get budget configuration link for any cloud
 *  @param Array params
 *                      Array containing the cloud_context and either the template_id or the instance_id
 *  @return String
 *                      The HTML link
 */
function _cloud_metering_get_budget_config_link($params = array()) {
  
  if (empty($params['cloud_context'])) {
    
    return '';
  
  }
  
  $cloud_context = $params['cloud_context'];
  
  if (isset($params['template_id'])) {
    $template_id = $params['template_id'];
    return l(t('Edit template budget'), CLOUD_SERVER_TEMPLATES_PATH . '/' . $cloud_context . '/' . $template_id . '/edit', array('fragment' => 'edit-initial-budget')) ;    
  }
  elseif (isset($params['instance_id'])) {
    $instance_id = $params['instance_id'];
    return l(t('Edit instance budget'), '/admin/settings/cloud_metering/threshold/' . $cloud_context . '/' . $instance_id) ;    
  }

  return '';  
  
}

/**
 *  Get user groups
 *  @param Array gids
 *                    Optional array of group-ids to return
 *  @return Array
 *                    The database rows corresponding to the groups
 */
function _cloud_metering_get_groups($gids = array()) {
  
  $return = array();
  
  $query = 'SELECT * FROM {' . CLOUD_METERING_BUDGET_GROUP_TABLE . '}';
  
  if (!empty($gids)) {
    $query .= ' WHERE gid IN (' . db_placeholders($gids) . ')';
  }
  
  $result = db_query($query, $gids);
  
  while ($row = db_fetch_array($result)) {
    $return[] = $row;
  }
  
  return $return;
}

/**
 *  Get group admins
 *  @param Integer gid The group-id
 *  @return Array The group admin(s)
 */
function _cloud_metering_get_group_admins($gid) {
  
  $query = 'SELECT user_data FROM {' . CLOUD_METERING_BUDGET_GROUP_TABLE . '} WHERE gid = %d';
  $result = db_result(db_query($query, $gid));
  
  if ($result !== FALSE) {
    $result = unserialize($result);
    if (is_array($result) && sizeof($result) > 0) {
      $return = array();
      foreach ($result as $uid => $value) {
        if ($value > 0) {
          $return[] = $uid;
        }
      }
      return $return;
    }
  } 
  
  return array(1); // Return the super-user in case there is no group admin(s)
  
}

/**
 * Filter a variable
 * @param String var
 *                  The variable to sanitize
 * @return String
 *                  The sanitized value
 */
function _cloud_metering_filter_input($var) {
  return filter_xss(check_plain($var));
}