<?php

/**
 * @file
 * Contains DB CRUD methods used by the Cloud Metering module
 * Works with Cloud Metering module
 * 
 * Copyright (c) 2011 DOCOMO Innovations, Inc.
 * 
 */

/**
 * Get cloud instance cost in given period of time
 * @param $start datetime Start date
 * @param $end datetime End Date
 * @param $cloud_context string The cloud name
 * @param $instance_id string The instance id
 * @return array The result of the database query
 */
 
function _cloud_metering_instance_cost_data($start, $end, $cloud_context = '', $instance_id = '') {

    if ($start && $end) {
      if ($end) {
        $now = date('c');
        $end = (strtotime($end) < time()) ? $end : $now;
      }

      $start_search     = "if (launch_time < '" . $start . "', '" . $start . "', launch_time) ";
      $end_search       = "if (instance_state_name != 'terminated', '" . $end . "', if (terminated_date < '" . $end . "' and terminated_date > '" . $start . "', terminated_date, '" . $end . "')) ";
      
      $search_where_sub = ' and launch_time < ' . $end_search . " and (terminated_date > '" . $start . "' or terminated_date = '0000-00-00 00:00:00' or terminated_date IS null) and instance_state_name != 'pending' ";
    }
    
    
    if ($cloud_context) {
      $search_where = " and a.cloud_type = '" . $cloud_context . "'";
    }
    else {
      $search_where = '';
    }
    
    if ($instance_id) {
      $search_instance = " and a.instance_id = '" . $instance_id . "'";
    }
    else {
      $search_instance = '';
    }
     
    //Construct a query for getting the Clound Instance Pricing details with total hours in running
    $query = 'SELECT a.instance_id, a.instance_type, a.instance_state_name, a.cloud_type, a.launch_time, b.linux_or_unix_usage, b.windows_usage ' . 
     'FROM  {' . CLOUD_BILLING_INSTANCES_DETAILS_TABLE . '} a LEFT JOIN {' . CLOUD_PRICING_INSTANCES_TABLE . '} b ON a.instance_type = b.instance_type where a.cloud_type = b.cloud_type ' . $search_instance . $search_where;
     
    $query_result = cloud_get_db_results($query);
           
    return $query_result[0];
}

/**
 *  Check if a notification has been sent against an instance
 */
function cloud_metering_notification_sent($type, $cloud_context = '', $instance_id = '', $uid = 0, $gid = 0) {
  
  $where  = '';
  $params = array();
  
  if (!empty($cloud_context) && !empty($instance_id)) {
    $where    = "cloud_name = '%s' AND instance_id = '%s'";
    $params[] = $cloud_context;
    $params[] = $instance_id;
  }
  elseif ($uid > 0) {
    $where    = 'uid = %d';
    $params[] = $uid;
  }
  elseif ($gid > 0) {
    $where    = 'gid = %d';
    $params[] = $gid;
  }
  
  $where   .= empty($where) ? '' : ' AND';
  $where   .= " type = '%s' AND timestamp > %d";
  $params[] = $type;
  $params[] = strtotime('-' . CLOUD_METERING_MAIL_INTERVAL . ' Days');
  
  $query    = 'SELECT COUNT(mid) FROM {' . CLOUD_METERING_MAILS_TABLE . '} WHERE ' . $where;
  
  $result   = db_result( db_query( $query, $params ) );
  
  return $result;
}

/**
 * Save notification mail sending info
 */
function cloud_metering_save_notification_info($type, $cloud_context = '', $instance_id = '', $uid = 0, $gid = 0) {
  $timestamp = time();
  $token = drupal_get_token($cloud_context . $instance_id . $timestamp);
  
  $query = 'INSERT INTO {' . CLOUD_METERING_MAILS_TABLE . "} (cloud_name, instance_id, uid, gid, timestamp, token, token_used, type) VALUES ('%s', '%s', %d, %d, %d, '%s', 0, '%s')";
  db_query($query, $cloud_context, $instance_id, $uid, $gid, $timestamp, $token, $type);
}

/**
 *  Delete metering data related to specific cloud
 */
function _cloud_metering_delete_cloud_data($cloud_context) {
  
  $query = 'DELETE FROM {' . CLOUD_METERING_INSTANCES_BUDGET_TABLE . "} WHERE cloud_context = '%s'";
  db_query($query, $cloud_context);
  
}

/**
 *  Update instance budget data
 *  @param string cloud_context
 *  @param string instance_id
 *  @param array data
 *                array containing the budget and the threshold
 */
function _cloud_metering_instance_budget_save($cloud_context, $instance_id, $data) {
  
  // Extra permission check
  if (!user_access('configure instance budget')) {
    drupal_set_message(t('You do not have permissions to set budget settings.'));
    return;
  }
  
  $budget    = (isset($data['budget']))    ? $data['budget'] : NULL;
  $threshold = (isset($data['threshold'])) ? $data['threshold'] : NULL;
  
  if (is_null($budget) && is_null($threshold)) {
    return;
  }
  
  $existing_data = _cloud_metering_get_instance_budget($cloud_context, $instance_id);
  
  $query  = '';
  $params = array();
  if (!is_null($existing_data['instance_id'])) {
    
    $query = 'UPDATE {' . CLOUD_METERING_INSTANCES_BUDGET_TABLE . '} SET ';
    
    $update_columns = '';
    
    if (!is_null($budget)) {
      $update_columns = 'initial_budget = %f';
      $params[] = $budget;
    }
    
    if (!is_null($threshold)) {
      $update_columns .= ', cost_threshold = %d';
      $params[] = $threshold;
    }
    
    $query .= $update_columns . " WHERE cloud_context = '%s' AND instance_id = '%s'";
    $params[] = $cloud_context;
    $params[] = $instance_id;
    
  }
  else {
    
    $budget    = is_null($budget) ? 0 : $budget;
    $threshold = is_null($threshold) ? 0 : $threshold;
    
    $query = 'INSERT INTO {' . CLOUD_METERING_INSTANCES_BUDGET_TABLE . "} (cloud_context, instance_id, initial_budget, cost_threshold) VALUES ('%s', '%s', %f, %d)";
    $params = array($cloud_context, $instance_id, $budget, $threshold);
    
  }
  
  $result = db_query($query, $params);
  
  if ($result) {
    drupal_set_message(t('The instance budget settings have been saved.')); 
  }
  else {
    drupal_set_message(t('There was an error while saving the instance budget settings. You may find more information about this error in the error log.'));
  }
  
}

/**
 *  Get instance budget data
 *  @param string cloud_context
 *  @param string instance_id
 *  @return array data
 *                  array containing instance_id, budget and threshold
 */
function _cloud_metering_get_instance_budget($cloud_context, $instance_id) {
  
  $return = array('instance_id' => NULL, 'budget' => 0, 'threshold' => 0);
  
  $query  = 'SELECT instance_id, initial_budget, cost_threshold FROM {' . CLOUD_METERING_INSTANCES_BUDGET_TABLE . "} WHERE cloud_context = '%s' AND instance_id = '%s'";
  $result = db_query($query, $cloud_context, $instance_id);
  $row    = db_fetch_array($result);
  
  if ($row) {
    $return['instance_id'] = $row['instance_id'];
    $return['budget']      = $row['initial_budget'];
    $return['threshold']   = $row['cost_threshold'];
  }
  
  return $return;  
}

/**
 *  Get template budget data
 *  @param int template_id
 *  @return array
 *                  array containing template_id, budget and threshold
 */
function _cloud_metering_get_template_budget($template_id) {
  
  $return = array('template_id' => NULL, 'budget' => 0, 'threshold' => 0);
  
  $query  = 'SELECT template_id, initial_budget, cost_threshold FROM {' . CLOUD_METERING_TEMPLATES_BUDGET_TABLE . '} WHERE template_id = %d';
  $result = db_query($query, $template_id);
  $row    = db_fetch_array($result);
  
  if ($row) {
    $return['template_id'] = $row['template_id'];
    $return['budget']      = $row['initial_budget'];
    $return['threshold']   = $row['cost_threshold'];
  }
  
  return $return;  
}

/**
 *  Update template budget
 *  @param int template_id
 *  @param array data
 *                      Array containing the budget and the threshold
 */
function _cloud_metering_template_budget_save($template_id, $data) {
  
  // User must have privilege to edit server templates
  
  
  $budget    = (isset($data['budget'])) ? $data['budget'] : NULL;
  $threshold = (isset($data['threshold'])) ? $data['threshold'] : NULL;
  
  if (is_null($budget) && is_null($threshold)) {
    return;
  }
  
  $existing_data = _cloud_metering_get_template_budget($template_id);
  
  $query  = '';
  $params = array();
  if (!is_null($existing_data['template_id'])) {
    
    $query = 'UPDATE {' . CLOUD_METERING_TEMPLATES_BUDGET_TABLE . '} SET ';
    
    $update_columns = '';
    
    if (!is_null($budget)) {
      $update_columns = 'initial_budget = %f';
      $params[] = $budget;
    }
    
    if (!is_null($threshold)) {
      $update_columns .= ', cost_threshold = %d';
      $params[] = $threshold;
    }
    
    $query .= $update_columns . ' WHERE template_id = %d';
    $params[] = $template_id;
    
  }
  else {
    
    $budget    = is_null($budget) ? 0 : $budget;
    $threshold = is_null($threshold) ? 0 : $threshold;
    
    $query = 'INSERT INTO {' . CLOUD_METERING_TEMPLATES_BUDGET_TABLE . '} (template_id, initial_budget, cost_threshold) VALUES (%d, %f, %d)';
    $params = array($template_id, $budget, $threshold);
    
  }
  
  $result = db_query($query, $params);
  
  if ($result) {
    drupal_set_message(t('The template budget settings have been saved.'));
    return TRUE;
  }
  else {
    drupal_set_message(t('There was an error while saving the template budget settings. You may find more information about this error in the error log.'));
    return FALSE;
  }
  
}

/**
 *  Get ssh key owner from db
 */
function _cloud_metering_get_key_owner($cloud_context, $key) {
  
  $ec2_lib_ssh_userkeys_table = aws_ec2_lib_get_table_info( AWS_EC2_LIB_USER_KEYS_TABLE ) ;
  $key_owner                  = db_result(db_query('SELECT owner FROM {' . $ec2_lib_ssh_userkeys_table . "} WHERE key_name = '%s' AND cloud_type = '%s'", $key, $cloud_context));
  
  return $key_owner;
  
}

/**
 *  Get server template data from db
 */
function _cloud_metering_get_template_data($template_id) {
  
  $result = db_query('SELECT * FROM {' . CLOUD_SERVER_TEMPLATES_TABLE . '} WHERE template_id = %d', $template_id);
  $row    = db_fetch_array($result);
  
  return $row;
  
}

/**
 *  Get users whose budgets have been configured
 */
function _cloud_metering_get_budget_configured_users() {
  
  $rows = array();
  
  $query  = 'SELECT * FROM {' . CLOUD_METERING_USER_BUDGET_TABLE . '} WHERE budget != 0';
  $result = db_query($query);
  
  while ($row = db_fetch_array($result)) {
    $rows[] = $row;
  }
  
  return $rows;
}