<?php

/**
 * @file
 * Contains the Queue handling methods used by the Cloud Metering module
 * Works with Cloud Metering module
 * 
 * Copyright (c) 2011 DOCOMO Innovations, Inc. 
 */

/**
 *  Implementation of hook_cron_queue_info()
 */
function cloud_metering_cron_queue_info() {
  
  $queues[CLOUD_METERING_QUEUE] = array(
    'worker callback' => '_cloud_metering_process_queue', 
    'time' => 600, 
  );
  
  return $queues;
}

/**
 *  Queue worker callback
 */
function _cloud_metering_process_queue() {
  
  while ( TRUE ) {
    
    $next_task = _cloud_metering_next_task_to_execute();
    
    if (is_array($next_task) && sizeof($next_task) > 0) {
      
      if ($next_task['task'] == CLOUD_METERING_BACKUP) {
        
        _cloud_metering_backup($next_task);
        continue;
      
      }
      elseif ($next_task['task'] == CLOUD_METERING_DETACH) {
        
        _cloud_metering_detach_volumes($next_task);
        continue;
        
      }
      elseif ($next_task['task'] == CLOUD_METERING_TERMINATE) {
        
        _cloud_metering_terminate($next_task);
        continue;
        
      }
      elseif ($next_task['task'] == CLOUD_METERING_BACKUP_CHECK) {
        
        _cloud_metering_check_backup($next_task);
        continue;
        
      }
      elseif ($next_task['task'] == CLOUD_METERING_DETACH_CHECK) {
        
        _cloud_metering_check_detach($next_task);
        continue;
        
      }
      elseif ($next_task['task'] == CLOUD_METERING_TERMINATE_CHECK) {
        
        _cloud_metering_check_terminate($next_task);
        continue;
        
      }
    }
    else {
      
      return;
      
    }
    
  }
  
}

/**
 *  Add item to Drupal queue
 */
function _cloud_metering_add_queue_item($item) {
  
  $queue = DrupalQueue::get(CLOUD_METERING_QUEUE) ;
  $queue->createItem($item) ;
  
}

/**
 *  Get the uuid
 */
function _cloud_metering_get_uuid($cloud_context, $instance_id) {
  
  return $cloud_context . '-' . $instance_id;
  
}

/**
 *  Check task progress and add new tasks if needed
 */
function _cloud_metering_check_task($task) {
  
  $qid  = $task['qid'];
  $data = $task['data'];
  $cloud_context = $data['cloud_context'];
  $instance_id = $data['instance_id'];
  $module = cloud_get_module($cloud_context);
  
  switch ($task['task']) {
    
    case CLOUD_METERING_BACKUP:
      
      $volumes = $data['volumes'];
      
      $backup_complete = FALSE;
      $snapshots_complete = array();
      
      
      foreach ($volumes as $volume_id => $volume) {
        
        if (!isset($volume['snapshot_id'])) {
          break;
        }
        
        $snapshot_id = $volume['snapshot_id'];
        $params      = array('cloud_context' => $cloud_context, 'snapshot_id' => $snapshot_id);
        $snapshot_complete = module_invoke($module, 'cloud_action', 'check_snapshot_completion', $params);
        
        if (!$snapshot_complete) {
          break;
        }
        
        $snapshots_complete[] = $snapshot_id;
        
      }
      
      $backup_complete = ( sizeof($volumes) == sizeof($snapshots_complete) );
      
      if ($backup_complete) {
        // Backup is complete. We mark the backup task as complete and start a detach job for the instance
        _cloud_metering_update_task_status($qid, CLOUD_METERING_QUEUE_STATUS_COMPLETED, array());
        // Start detach task, send in the volume information already in $data
        _cloud_metering_add_task($cloud_context, $instance_id, CLOUD_METERING_DETACH, $data);
      }
      
      break;
    
    case CLOUD_METERING_DETACH:
      
      $volumes = $data['volumes'];      
      $detach_complete = FALSE;
      $volumes_detached = array();
      
      foreach ($volumes as $volume_id => $volume) {
        
        $params = array('cloud_context' => $cloud_context, 'volume_id' => $volume_id);
        $volume_attached = module_invoke($module, 'cloud_action', 'check_volume_attached_status', $params);
        
        if ($volume_attached) {
          break;
        }
        
        $volumes_detached[] = $volume_id;
        
      }
      
      $detach_complete = ( sizeof($volumes) == sizeof($volumes_detached) );
      
      if ($detach_complete) {
        // Set the detach task as complete
        _cloud_metering_update_task_status($qid, CLOUD_METERING_QUEUE_JOB_COMPLETED, array());
        // Start a terminate task for the instance
        // _cloud_metering_add_task($cloud_context, $instance_id, CLOUD_METERING_TERMINATE, $data);
      }
      
      break;
    
    case CLOUD_METERING_TERMINATE:
      
      // Check if the instance is terminated
      $params = array('cloud_context' => $cloud_context, 'instance_id' => $instance_id);
      $terminated = module_invoke($module, 'cloud_action', 'check_instance_terminated', $params);
      
      if ($terminated) {
        // Termination complete
        _cloud_metering_update_task_status($qid, CLOUD_METERING_QUEUE_JOB_COMPLETED, array()); 
      }
      
      break;
    
  }
  
}

/**
 *  Backup instance volumes
 */
function _cloud_metering_backup($task) {
  
  $qid = $task['qid'];
  $data = $task['data'];
  $volumes = $data['volumes'];
  $cloud_context = $data['cloud_context'];
  $instance_id = $data['instance_id'];
  
  // Create snapshots for each volume
  foreach ($volumes as $volume_id => $volume) {
  
    $params = array(
      'volume_id'            => $volume_id,
      'snapshot_name'        => 'Auto-Snapshot ' . date('YmdHis'), 
      'snapshot_description' => t('Snapshot created before terminating instance as instance crossed the allocated budget'),
    );
    
    $snapshot_id = cloud_perform_action('', 'backup', $cloud_context, $params);
    
    if ($snapshot_id) {
      $volumes[$volume_id]['snapshot_id'] = $snapshot_id;
    }
    else {
      // TODO -- Decide on steps to be taken in case snapshot creation fails
    }
    
  }
  
  $data['volumes'] = $volumes;
  
  // The backup task will be on a wait as we keep monitoring for the backup to complete
  _cloud_metering_update_task_status($qid, CLOUD_METERING_QUEUE_STATUS_IN_PROGRESS, $data);
  
  // We will start a checker task in next cron run
  
}

/**
 *  Detach volumes
 */
function _cloud_metering_detach_volumes($task) {
  
  $qid = $task['qid'];
  $data = $task['data'];
  $volumes = $data['volumes'];
  $cloud_context = $data['cloud_context'];
  $instance_id = $data['instance_id'];
  
  // Start detach for each volume
  foreach ($volumes as $volume_id => $volume) {
  
    $params = array(
      'volume_id'     => $volume_id,
    );
    
    $result = cloud_perform_action('', 'detach_volume', $cloud_context, $params);    
    
    if (!$result) {
      _cloud_metering_activity_audit(t('Detaching of volume @volume-id failed.', array('!volume-id' => $volume_id)), 'user_activity', '', WATCHDOG_ERROR);
    }
    
  }
  
  // Detach task will be on wait till the detachings are completed
  _cloud_metering_update_task_status($qid, CLOUD_METERING_QUEUE_STATUS_IN_PROGRESS); 
  
}

/**
 *  Terminate instance
 */
function _cloud_metering_terminate($task) {
  
  $qid = $task['qid'];
  
  $data = $task['data'];
  $cloud_context = $data['cloud_context'];
  $instance_id = $data['instance_id'];
  
  // DEBUG -- Terminate is not being continued
  drupal_set_message(t('Terminate request for instance %instance-id', array('%instance-id' => $instance_id)));
  _cloud_metering_update_task_status($qid, CLOUD_METERING_QUEUE_STATUS_COMPLETED, array());
  return;
  
  $params = array(0 => $instance_id, 'instance_id' => $instance_id);
  
  // Start termination of instance
  cloud_perform_action('', 'terminate', $cloud_context, $params);
  
  // Update the task status. Terminate task will be waiting for the termination to complete.
  _cloud_metering_update_task_status($qid, CLOUD_METERING_QUEUE_STATUS_WAITING);
  
}

/**
 *  Check backup task. If completed then mark as completed. Otherwise start a new check task.
 */
function _cloud_metering_check_backup($task) {
  
  $qid = $task['qid'];
  $data = $task['data'];
  $b_qid = $data['qid'];
  $cloud_context = $data['cloud_context'];
  $instance_id = $data['instance_id'];
  $volumes = $data['volumes'];
  
  $snapshots_complete = array();
  
  foreach ($volumes as $volume_id => $volume) {
    
    $snapshot_id = $volume['snapshot_id'];
    $params      = array('snapshot_id' => $snapshot_id);
    $snapshot_complete = cloud_perform_action('', 'check_snapshot_completion', $cloud_context, $params);
    
    if (!$snapshot_complete) {
      break;
    }
    
    $snapshots_complete[] = $snapshot_id;
    
  }
  
  if (sizeof($volumes) == sizeof($snapshots_complete)) {
    
    // Backup of all volumes completed
    _cloud_metering_update_task_status($b_qid, CLOUD_METERING_QUEUE_STATUS_COMPLETED, array());
    
    // Initiate a detach task
    _cloud_metering_add_task($cloud_context, $instance_id, CLOUD_METERING_DETACH, $data);
  }
  else {
    $backup_task = _cloud_metering_get_task($b_qid);
    
    if ($backup_task['time_running'] > 10) {
      _cloud_metering_update_task_status($b_qid, CLOUD_METERING_QUEUE_STATUS_FAILED, array());
      _cloud_metering_activity_audit(t('Task with %qid failed due to expiry of allocated time.', array('%qid' => $b_qid)), '', WATCHDOG_ERROR);
    }
    
  }
  
  // The current check task is completed
  _cloud_metering_update_task_status($qid, CLOUD_METERING_QUEUE_STATUS_COMPLETED, array());
  
}

/**
 *  Check detach task. If completed, mark as completed. Otherwise keep monitoring.
 */
function _cloud_metering_check_detach($task) {
  
  $qid = $task['qid'];
  $data = $task['data'];
  $d_qid = $data['qid']; // Id of task being monitored
  $cloud_context = $data['cloud_context'];
  $instance_id = $data['instance_id'];
  $volumes = $data['volumes'];
  
  $detach_complete = FALSE;
  $volumes_detached = array();
  
  foreach ($volumes as $volume_id => $volume) {
    
    $params          = array('volume_id' => $volume_id);
    $volume_attached = cloud_perform_action('', 'check_volume_attached_status', $cloud_context, $params);
    
    if ($volume_attached) {
      break;
    }
    
    $volumes_detached[] = $volume_id;
    
  }
  
  $detach_complete = ( sizeof($volumes) == sizeof($volumes_detached) );
  
  if ($detach_complete) {
    
    // Mark detach task as completed and start a terminate task
    _cloud_metering_update_task_status($d_qid, CLOUD_METERING_QUEUE_STATUS_COMPLETED, array());
    _cloud_metering_add_task($cloud_context, $instance_id, CLOUD_METERING_TERMINATE, $data);
    
  }
  else {
    
    // If task ran for more than stipulated time, cancel it
    $d_task = _cloud_metering_get_task($d_qid);
    
    if ($d_task['time_running'] > 10) {
      _cloud_metering_update_task_status($d_qid, CLOUD_METERING_QUEUE_STATUS_FAILED, array());
      _cloud_metering_activity_audit(t('Task with Id %qid failed due to expiry of allocated time.', array('%qid' => $d_qid)), 'user_activity', '', WATCHDOG_ERROR);
      // We proceed to terminate instance even if detaching failed
      _cloud_metering_add_task($cloud_context, $instance_id, CLOUD_METERING_TERMINATE, $data);
    }
    
  }
  
  _cloud_metering_update_task_status($qid, CLOUD_METERING_QUEUE_STATUS_COMPLETED, array());
  
}

/**
 *  Monitor termination of instances
 */
function _cloud_metering_check_terminate($task) {
  
  $qid = $task['qid'];
  $data = $task['data'];
  $t_qid = $data['qid'];
  $cloud_context = $data['cloud_context'];
  $instance_id = $data['instance_id'];
  
  // Check if instance has got terminated
  $params     = array('instance_id' => $instance_id);
  $terminated = cloud_perform_action('', 'check_instance_terminated', $cloud_context, $params);
  
  if ($terminated) {
    // Termination task completed
    _cloud_metering_update_task_status($t_qid, CLOUD_METERING_QUEUE_STATUS_COMPLETED, array());
  }
  else {
    $t_task = _cloud_metering_get_task($t_qid);
    if ($t_task['time_running'] > 10) {
      _cloud_metering_update_task_status($t_qid, CLOUD_METERING_QUEUE_STATUS_FAILED, array());
      _cloud_metering_activity_audit(t('Task with Id %qid failed due to expiry of allocated time.', array('%qid' => $t_qid)), 'user_activity', '', WATCHDOG_ERROR);
    }
  }
  
  // This task is complete
  _cloud_metering_update_task_status($qid, CLOUD_METERING_QUEUE_STATUS_COMPLETED, array());
}

/**
 *  Get the base tasks - backup, detach, terminate
 */
function _cloud_metering_get_base_tasks() {
  
  return array(CLOUD_METERING_BACKUP, CLOUD_METERING_DETACH, CLOUD_METERING_TERMINATE);
  
}

/**
 *  Get checker tasks - backup check, detach check, terminate check
 */
function _cloud_metering_get_checker_tasks() {
  
  return array(CLOUD_METERING_BACKUP_CHECK, CLOUD_METERING_DETACH_CHECK, CLOUD_METERING_TERMINATE_CHECK);
  
}

/**
 *  Get the type of a task
 *  @param String task
 *  @return String "base" or "checker"
 */
function _cloud_metering_get_task_type($task) {
  
  $base_tasks = _cloud_metering_get_base_tasks() ;
  if (in_array($task, $base_tasks)) {
    return 'base' ;
  }
  
  $checker_tasks = _cloud_metering_get_checker_tasks() ;
  if (in_array($task, $checker_tasks)) {
    return 'checker' ;
  }
  
  return 'base' ;
  
}