<?php

/**
 * @file
 * Contains the DB CRUD methods for Queue handling used by the Cloud Metering module
 * Works with Cloud Metering module
 * 
 * Copyright (c) 2011 DOCOMO Innovations, Inc.
 * 
 */

/**
*  Add a task
*/
function _cloud_metering_add_task($cloud_context, $instance_id, $task, $data = array()) {
  
  drupal_queue_include();
  
  $uuid      = _cloud_metering_get_uuid($cloud_context, $instance_id);
  $task_type = _cloud_metering_get_task_type($task);
  
  if (!_cloud_metering_uuid_in_queue($uuid, $task_type)) {
    
    $data['cloud_context'] = $cloud_context;
    $data['instance_id'] = $instance_id;
    
    $query = 'INSERT INTO {' . CLOUD_METERING_QUEUE_TABLE . "} VALUES (NULL, '%s', '%s', CURRENT_TIMESTAMP, CURRENT_TIMESTAMP, '%s', %d, '%s')";
    // The user id is hard-coded to user-1
    $result = db_query($query, $task, $uuid, CLOUD_METERING_QUEUE_STATUS_INITIATED, 1, serialize($data));
    if ($result) {
      // Add item to queue
      _cloud_metering_add_queue_item($task);
      _cloud_metering_activity_audit(t('@task task initiated for instance %instance_id', array('@task' => ucfirst($task), '%instance_id' => $instance_id)), 'user_activity');
    }
    
  }
  
}

/**
 *  Check if a base task is running on an instance
 *  @param String uuid
 *                      The unique id of associated with a task
 *  @param String type
 *                      The type of task to check - base or checker
 */
function _cloud_metering_uuid_in_queue($uuid, $type = 'base') {
  
  $tasks_to_check = ($type == 'base' ? _cloud_metering_get_base_tasks() : _cloud_metering_get_checker_tasks()) ;
  
  $query = 'SELECT COUNT(uuid) AS added FROM {' . CLOUD_METERING_QUEUE_TABLE . "}
              WHERE uuid = '%s' AND status IN ('" . CLOUD_METERING_QUEUE_STATUS_INITIATED . "', '" . CLOUD_METERING_QUEUE_STATUS_IN_PROGRESS . "')
              AND task IN (" . db_placeholders($tasks_to_check, 'varchar') . ')';
  
  $params = array($uuid);
  $params = array_merge($params, $tasks_to_check);
  $count  = db_result(db_query($query, $params));
  
  return ($count ? TRUE : FALSE);
  
}

/**
 *  Get the next queue task to execute
 */
function _cloud_metering_next_queue_task() {
  
  $query = 'SELECT * FROM {' . CLOUD_METERING_QUEUE_TABLE . "} WHERE status = '%s' ORDER BY created";
  
  $result = db_query_range($query, CLOUD_METERING_QUEUE_JOB_INITIATED, 0, 1);
  
  $task = array();
  
  while ($o = db_fetch_object($result)) {
    
    $task['qid']       = $o->qid;
    $task['task']      = $o->task;
    $task['uuid']      = $o->uuid;
    $task['timestamp'] = $o->timestamp;
    $task['status']    = $o->status;
    $task['uid']       = $o->uid;
    
  }
  
  return $task;
}

/**
 *  Update status of queue item
 */
function _cloud_metering_update_task_status($qid, $status, $data = NULL) {
  
  $params = array();
  if ( is_array($data) ) {
    $data = (sizeof($data) > 0) ? serialize($data) : '';
    $params = array($status, $data, $qid);
    $query = 'UPDATE {' . CLOUD_METERING_QUEUE_TABLE . "} SET status = '%s', updated = CURRENT_TIMESTAMP, data = '%s' WHERE qid = %d";
  }
  else {
    $params = array($status, $qid);
    $query = 'UPDATE {' . CLOUD_METERING_QUEUE_TABLE . "} SET status = '%s', updated = CURRENT_TIMESTAMP WHERE qid = %d";
  }
  $result = db_query($query, $params);
  
  return ($result ? TRUE : FALSE);
}

/**
 * Get the task in progress
 */
function _cloud_metering_next_task_in_progress() {
  
  $query = 'SELECT * , TIMESTAMPDIFF(MINUTE, updated, CURRENT_TIMESTAMP) time_running FROM {' . CLOUD_METERING_QUEUE_TABLE . "} WHERE status = '%s' ORDER BY created ASC LIMIT 0, 1" ;
  $db_result = db_query($query, CLOUD_METERING_QUEUE_STATUS_IN_PROGRESS);    
  
  $task = array() ;
  
  while ($o = db_fetch_object($db_result) ) {
    
    $task['qid']          = $o->qid;
    $task['task']         = $o->task;
    $task['uuid']         = $o->uuid;
    $task['created']      = $o->created;
    $task['updated']      = $o->updated;
    $task['status']       = $o->status;
    $task['uid']          = $o->status;
    $task['data']         = unserialize($o->data);
    $task['time_running'] = $o->time_running;
    
  }

  return $task ;
  
}

/**
 *  Check if a task of specific type is running
 */
function _cloud_metering_task_running($cloud_context, $instance_id, $task) {
  
  $query = 'SELECT COUNT(qid) AS task_count FROM {' . CLOUD_METERING_QUEUE_TABLE . "} WHERE uuid = '%s' AND task = '%s' AND status IN ('" . CLOUD_METERING_QUEUE_STATUS_INITIATED . "', '" . CLOUD_METERING_QUEUE_STATUS_IN_PROGRESS . "')";
  
  $result = db_result(db_query($query, _cloud_metering_get_uuid($cloud_context, $instance_id), $task));
  
  return ($result ? TRUE : FALSE);
}

/**
 *  Get next base task to execute
 *  @return
 *          The next task to execute. Next task is either the oldest initiated base task,
 *          or the oldest initiated checker task
 */
function _cloud_metering_next_task_to_execute() {
  
  $base_tasks    = _cloud_metering_get_base_tasks();
  $checker_tasks = _cloud_metering_get_checker_tasks();
  
  $query = 'SELECT * FROM {' . CLOUD_METERING_QUEUE_TABLE . "} WHERE status = '" . CLOUD_METERING_QUEUE_STATUS_INITIATED . "' ORDER BY created ASC LIMIT 0, 1" ;
  
  $db_result = db_query($query);    
  
  $task = array() ;
  
  while ($o = db_fetch_object($db_result) ) {
    
    $task['qid']          = $o->qid;
    $task['task']         = $o->task;
    $task['uuid']         = $o->uuid;
    $task['created']      = $o->created;
    $task['updated']      = $o->updated;
    $task['status']       = $o->status;
    $task['uid']          = $o->status;
    $task['data']         = unserialize($o->data);
    
  }
  
  return $task ;
  
}

/**
 *  Get a task from queue database table
 */
function _cloud_metering_get_task($qid) {
  
  
  $query = 'SELECT *, TIMESTAMPDIFF(MINUTE, updated, CURRENT_TIMESTAMP) time_running FROM {' . CLOUD_METERING_QUEUE_TABLE . '} WHERE qid = %d' ;
  $db_result = db_query($query, $qid);    
  
  $task = array() ;
  
  while ( $o = db_fetch_object($db_result) ) {
    
    $task['qid']          = $o->qid;
    $task['task']         = $o->task;
    $task['uuid']         = $o->uuid;
    $task['created']      = $o->created;
    $task['updated']      = $o->updated;
    $task['status']       = $o->status;
    $task['uid']          = $o->status;
    $task['data']         = unserialize($o->data);
    $task['time_running'] = $o->time_running;
  }
  
  return $task ;
  
}

/**
 *  Get the tasks having status "in progress" and have run for atleast a minute
 */
function _cloud_metering_get_in_progress_tasks($type = 'base') {
  
  $task_types = ($type == 'base' ? _cloud_metering_get_base_tasks() : _cloud_metering_get_checker_tasks());
  
  $tasks = array();
  
  $query  = 'SELECT * FROM {' . CLOUD_METERING_QUEUE_TABLE . "} WHERE status = '%s' AND task IN (" . db_placeholders($task_types, 'varchar') . ') AND TIMESTAMPDIFF(MINUTE, updated, CURRENT_TIMESTAMP) >= 1 ORDER BY created ASC' ;
  $params = array(CLOUD_METERING_QUEUE_STATUS_IN_PROGRESS);
  $params = array_merge($params, $task_types);
  $result = db_query($query, $params) ;
  
  while ($row = db_fetch_array($result)) {
    $tasks[] = $row;
  }
  
  return $tasks;
  
}

/**
 *  Remove the complete tasks from table
 */
function _cloud_metering_delete_completed_tasks() {
  
  $query = 'DELETE FROM {' . CLOUD_METERING_QUEUE_TABLE . "} WHERE status = '" . CLOUD_METERING_QUEUE_STATUS_COMPLETED . "'";
  db_query($query);
  
}