<?php

/**
 * @file
 * This class defines the neccessary methods needed 
 * to hook up a collectd monitoring instance within
 * XCP
 */
require_once drupal_get_path('module', 'cloud_monitoring') . '/plugins/cloud_monitoring_collectd.inc';

class xcp_collectd_monitor extends collectd_monitor {
  
  /**
	 * Set the endpoint for this monitor server
   */
  protected function set_endpoint() {
    if ($this->monitor_id) {
      $monitor = cloud_monitoring_load_monitor($this->monitor_id);
      if ($monitor) {
        $monitor_server = db_result(db_query("SELECT vm_ip from {cloud_xcp_instances} where vmid = '%s'", $monitor['instance_id']));
        if ($monitor_server) {
          if (!preg_match("/^(http|https)/i", $monitor_server)) {
            $monitor_server = 'http://' . $monitor_server;
          }
          $this->endpoint = $monitor_server;
        }
      }
    } 
  }
  
  /**
   * Method generates a small graph url
   */
  public function get_small_graph($host, $seconds, array $params) {
    $url = '';
    if (count($params)) {
      $params['q'] = 'monitor-server/get-graph-by-plugin';
      $params['host'] = $host;
      $params['seconds'] = $seconds;
      $params['image_preset'] = 'medium';
      $url = $this->build_url($params);
    }
    $instance = $this->get_vmid_by_host($host);
    
    $detail_path = '';
    if ($instance) {
      $details_url = "clouds/xcp/describe/{$instance}/graph-details";
      $details_params = $params;
      unset($details_params['q']);
      $details_params['image_preset'] = 'large';
    }
    return l("<img src={$url} class='graph-image'>", $details_url, array('html' => TRUE, 'query' => $details_params));
  }
  

  
  /**
   * Get all the hosts from a monitoring system
   */
  public function get_hosts() {
    $data = $this->execute(array('q' => 'monitor-server/get-all-hosts'));
    $hosts = array();
    if (count($data)) {
      foreach ($data as $value) {
        if ($this->get_monitor_instance_by_host($value) != FALSE) {
          $hosts[$value] = $value;
        }
      }
    }
    return $hosts;
  }
  
  /**
   * Get the list of servers
   * 
   */
  public function get_server_list() {
    $servers = array();
    if (isset($this->monitor_id)) {
      $query = "SELECT instance_id, status, vm_name, vmid from {cloud_xcp_instances} INNER JOIN {cloud_monitor_instance}
      on cloud_xcp_instances.vmid = cloud_monitor_instance.instance_id where monitor_id = %d order by vm_name";
      $result = db_query($query, $this->monitor_id);
      while ($obj = db_fetch_object($result)) {
        $servers[] = array(
          $obj->vm_name,
          $obj->vmid,
          ($obj->status) ? t('Enabled') : t('Disabled'),
        );
      }
    }
    return $servers;
  }
  
  /**
   * Load the details of this monitor
   * 
   */
  public function get_monitor_details() {
    $query = "SELECT cloud_monitor.monitor_id, cloud_monitor.monitor_name, cloud_monitor.monitor_type, cloud_monitor.instance_id, vm_name as nickname, vm_ip as dns_name from {cloud_monitor} 
    					INNER JOIN {cloud_xcp_instances} on cloud_monitor.instance_id = cloud_xcp_instances.vmid where monitor_id = %d";
    $result = db_query($query, array($this->monitor_id));
    return db_fetch_array($result);
  }
  
  public function get_monitor_instance_by_host($host) {
    $query = "SELECT instance_id from {cloud_monitor_instance} INNER JOIN {cloud_xcp_instances} 
    					where cloud_monitor_instance.instance_id = cloud_xcp_instances.vmid and vm_ip = '%s'";
    return db_result(db_query($query, $host));  
  }
  
  /**
   * Helper function to get the vm_ip by host
   */
  private function get_vmid_by_host($host) {
    $query = "SELECT vmid from {cloud_xcp_instances} where vm_ip = '%s'";
    return db_result(db_query($query, $host));
  }
  
  public function install_monitor_script(array $params) {
    return FALSE;
  }
  
  public function stop_monitor(array $params) {
    return FALSE;
  }
  
  public function start_monitor(array $params) {
    return FALSE;
  }
  
  public function delete_monitor(array $params) {
    return FALSE;
  }
}